package benchmarks;

import java.io.*;

public class BinaryTree implements Serializable {
    private int value = 0;
    private BinaryTree left = null;
    private BinaryTree right = null;
    
    public BinaryTree(BinaryTree left, BinaryTree right, int value) {
        super();
        this.left = left;
        this.right = right;
        this.value = value;
    }
    
    public BinaryTree(int value) {
        super();
        this.value = value;
    }
    
    public boolean isLeaf() { return this.left == null && this.right == null; }
    
    public void inc() {
        this.value++;
        if (this.left != null) { this.left.inc(); }
        if (this.right != null) { this.right.inc(); }
    }
    
    public void printNode() {
        if (this.isLeaf()) {
            System.out.print("leaf=" + this.value);
        } else {
            System.out.print("node=<" + this.value + ">");
        }
    }
    
    public void print() {
        if (this.isLeaf()) {
            System.out.print("leaf=" + this.value);
        } else {
            System.out.print("node=<" + this.value + ", left:(");
            this.left.print();
            System.out.print("), right:(");
            this.right.print();
            System.out.print(")>");
        }
    }
    
    public void println() {
        this.print();
        System.out.println();
    }
    
    public static BinaryTree createDepth(int depth) {
        BinaryTree left = null;
        BinaryTree right = null;
        BinaryTree res = null;
        if (depth == 0) {
            res = new BinaryTree(0);
        } else {
            left = BinaryTree.createDepth(depth - 1);
            right = BinaryTree.createDepth(depth - 1);
            res = new BinaryTree(left, right, 0);
        }
        return res;
    }
    
    final public static String jlc$CompilerVersion$jl = "2.3.0";
    final public static long jlc$SourceLastModified$jl = 1238584128874L;
    final public static String jlc$ClassType$jl =
      ("H4sIAAAAAAAAALVYXWxURRSe3W13u+1Cu6VAoaWlpUQa6JYY/7APUiuVwkJr" +
       "W4jUkjJ77+zube/e\ne70zu90SJKAJIA8aA/gXgRcTo+FBIeqLURPBf2OCCf" +
       "gCLxglUYy+KDFoPDP37t7du0tJSUoyw+zM\nmTNnzvnOd+b2zA1USU0UoYRS" +
       "RdcmI2zGIFT0emySSIxGRrYMYZMSuU/FlI7CwoS05z9l53vhZzd7\nkWcM1W" +
       "t6r6pgOpo09XQiOZpUaNZEKw1dnUmoOrM1luh4eNWt6W8Pb2nyodoxVKtoIw" +
       "wzRerTNUay\nbAyFUiQVIybtlWUij6GwRog8QkwFq8peENQ1OJgqCQ2ztEno" +
       "MKG6muGC9TRtEFOcmZuMopCka5SZ\naYnpJmWoLjqJM7g7zRS1O6pQ1hNF/r" +
       "hCVJk+jfYjbxRVxlWcAMEl0dwtuoXG7n4+D+LVCphpxrFE\nclsqphRNZqjV" +
       "vSN/446tIABbAynCknr+qAoNwwSqt0xSsZboHmGmoiVAtFJPwykMLb+tUhCq" +
       "MrA0\nhRNkgqFGt9yQtQRSQeEWvoWhxW4xoQlittwVs4JoDfpD/x4d+nslRB" +
       "xslomkcvv9sKnFtWmYxIlJ\nNIlYG2+mI8cHdqWbvQiB8GKXsCXTu/rDHdHr" +
       "n7RaMk1lZAYFFiekWw80r7jY+1PQx82oMnSqcCgU\n3VxEdche6ckagO4leY" +
       "18MZJb/HT4810H3iG/elHVAPJLuppOaQMoSDS5zx4HYBxVNGLNDsbjlLAB\n" +
       "VKGKKb8ufoM74opKuDsqYWxglhTjrIEQCkDzQJOR9Y//RgwteFTRsDkzahIS" +
       "oZMMtTNCGR2npjQe\nA9clU9icouNFQlmudOG0xwP3aXbnlgpA3KyrMjEnpL" +
       "eufb1v09bnj1iR4uiyzYHAO8ojjnLk8Qit\nizgCLQ/1miae4ZmRPXhxxWtf" +
       "4JPgb7g3VfYScS3vdAXvYdO9sxJHn5N2AzDCgIoJqeHA9eWv//D2\nBS/ylS" +
       "WPaH6yXzdTWOXxz2VLvX2cewVg0+EGb7mzfz+67dylb66scWDMUEdJdpXu5N" +
       "nR7va6qUtE\nBvZx1L/yz+Y/jlVueN+LKiDlgHQYBqhABre4zyjKkp4c4/C7" +
       "+KKoJl5y8WoGDpouuDDvQ2JcC6Go\n4uGAVm/jbAHv+GKYd/UWeCBcDa47CD" +
       "67+Zx//eWPai4Ip+Sor7aAI0cIsxIp7ECEIwfmr7w6dOzE\njcNP+SB1DUPA" +
       "wsOQ30jHVEXKwpalxbnHbyBzZP12tqfuhS76gUBBUEml0gzHVAJcjVVVnyby" +
       "BBNk\nFS4gRsFH4KxQDHgNKHJCBUWWOwxPBhBchjkijQ3HX+584zLnFkP4aw" +
       "nY6BOW+oRbcr5xE9mQqaSA\nJzI2kb3U8ubP564NN1jQsdh+VQnhFu6xGF/4" +
       "vsbgvmib7QQhfX5t25n9w1djFhPWF/tuk5ZO3X/6\nR9K5MSSVyW4f1KSsuO" +
       "FqD+8787dF4rZIWLKCp3qDE0egtykrIqHOkd1b9hxpF6F0+4d3XZb2+2ZZ\n" +
       "ayxB5FIbkQ3lEMm7trImeywoFaie7VIPegos85TYas9k3eEHT6yblb/6+aPA" +
       "SW9l318Hz35/IuRF\n3jEUUGg/cKjK0Uu3W4RVphy6VOz9eMepm9+xqyJ+Dk" +
       "Nww5qzpcy+ExeQ10OXMmH/u6dTXhQYQ3Xi\nSYM1thOraZ6JY/AooX32ZBQt" +
       "KFovfmBY1bQnz4DNbhgXHOvmJgdzMObSfFzloqOgHfigHfygK/ge\nj8EHm8" +
       "SGVtG358kjYJhKBvNXIKrMcONF9ARekBXnjUUHrYRWbR9UXf6grbx7nEHlJn" +
       "HmABehMura\noNXY6mrKq9tuq6s0lUSySF+2ANHIwasNQzFexOz0U/RI/kEL" +
       "zMcJYsXtHmCCHA4/+WfoED6/22uf\n081QkOlGl0oyRM1j2y/O8oO2rlmxvU" +
       "08Qx10+UYe6bxnXc0vULtuU5bD9uQwgTe3NpqLvajXeM71\nutV1V7c94UzT" +
       "E76k8qVXoM4CasnrunhTTzE8q81iQ4XTmooYqhHaMjvYy+bGUDn+uSM5jUM9" +
       "BPcR\nHBfLaw1rSwSQHtN1lWBNbJYKkIKypT92FRnOC32LbXjLPBmeFDVFcl" +
       "tdkdEVWWzT52Qy93WXbXLX\nPJnMICWAPjS2XZdF0PW7MJSXqg22oRvmydBn" +
       "gD2EoXdrJFc4YBs5ME9GHrLIWGOqdjdm8rYe2qBt\n5mA5M0tLgJePOyBpqP" +
       "iLQHkz5/Y+eJGhGskkUFQeI4b1fbbQ4xx/p3cDQ9XO5xJ/BzeW/HHD+gSX\n" +
       "2i/uWfOZEf5KfADkP5MD8K0aT6tqYeksGPsNk8QVYWjAKqSG+O8knOt8sfGZ" +
       "U0auhNSJEsKrecSq\n5ln0P6HrCR/LEQAA");
}
