package nbody;

import java.util.*;
import nbody.ordinary.*;

public class Sequential {
    
    public void run(boolean debug, int numParticles, int reps) {
        Particle[] particles = new Particle[numParticles];
        ParticleV[] pvs = new ParticleV[numParticles];
        this.initParticles(debug, particles, pvs);
        int i = 0;
        long timeStarted = 0;
        long timeFinished = 0;
        timeStarted = System.nanoTime();
        for (; i < reps; i++) {
            if (debug) {
                System.out.println("\nIteration: " + i);
                System.out.println("Particles: " + Arrays.toString(particles));
            }
            Worker.computeForces(particles, particles, pvs);
            Worker.computeNewPos(particles, pvs, i);
        }
        timeFinished = System.nanoTime();
        System.out.println("time = " + (timeFinished - timeStarted) / 1000 +
                           " micros");
        if (debug) {
            System.out.println("\nIteration: " + i);
            System.out.println("Particles: " + Arrays.toString(particles));
        }
    }
    
    private void initParticles(boolean debug, Particle[] particles,
                               ParticleV[] pvs) {
        for (int i = 0; i < particles.length; i++) {
            Particle p = new Particle();
            if (debug) {
                p.x = i;
                p.y = i;
                p.m = 1.0;
            } else {
                p.x = 10.0 * Math.random();
                p.y = 10.0 * Math.random();
                p.m = 10.0 * Math.random();
            }
            particles[i] = p;
            ParticleV pv = new ParticleV();
            pv.vi_old = 0;
            pv.vj_old = 0;
            pv.ai_old = 0;
            pv.aj_old = 0;
            pv.ai = 0;
            pv.aj = 0;
            pvs[i] = pv;
        }
    }
    
    public static void main(String[] args) {
        boolean debug = Boolean.parseBoolean(args[0]);
        int numParticles = Integer.parseInt(args[1]);
        int reps = Integer.parseInt(args[2]);
        Sequential s = new Sequential();
        s.run(debug, numParticles, reps);
    }
    
    public Sequential() { super(); }
    
    final public static String jlc$CompilerVersion$jl = "2.3.0";
    final public static long jlc$SourceLastModified$jl = 1238689332580L;
    final public static String jlc$ClassType$jl =
      ("H4sIAAAAAAAAAK1YXWxURRSe/en2b6HdUirSP6A10ABbkWjQarBpaGhZ0tKW" +
       "IqVkmb07u73t3Xsv\n987dblEJYiJojIlpVTQCLyYGwoNC1AeNmgD++4IJ+A" +
       "IvGCVRjDwYiUHjmZn9uXt3u4ChyczOnTlz\n5vx855yZnrqOykwDBU1imrKm" +
       "TgbpjE5M3muRSSJRMzjcP4gNk0R7FGyaI7AQlvb8K4++Hzi42Y1c\nY6hO1b" +
       "oVGZsjE4ZmxSdGJmQzZaBluqbMxBWNpjkW8His7db0d4f6Gz2oZgzVyOowxV" +
       "SWejSVkhQd\nQ/4ESUSIYXZHoyQ6hgIqIdFhYshYkfcBoabCwaYcVzG1DGIO" +
       "EVNTkoywzrR0YvAzM5Mh5Jc01aSG\nJVHNMCmqDU3iJO60qKx0hmSTdoWQLy" +
       "YTJWruRfuRO4TKYgqOA2FDKKNFJ+fY2cvmgbxKBjGNGJZI\nZot3SlajFLU6" +
       "d2Q1bt8CBLC1PEHohJY9yqtimEB1QiQFq/HOYWrIahxIyzQLTqFo6bxMgahC" +
       "x9IU\njpMwRUucdINiCagquVnYFooWO8k4J/DZUofPbN4a8Pn/eWnwr2XgcZ" +
       "A5SiSFye+DTS2OTUMkRgyi\nSkRsvGkF5/p2Wk1uhIB4sYNY0HQ/8NH20LXP" +
       "WgVNYxGaAY7FsHTrkabmC90/VXqYGBW6ZsoMCnma\nc68Ople6UjqguyHLkS" +
       "0GM4ufD32x88BJ8qsbVfQhn6QpVkLtQ5VEjfakx+UwDskqEbMDsZhJaB/y\n" +
       "KnzKp/FvMEdMVggzRxmMdUwn+DilI4TKobmgYST+qlhH0YJhstciKgU0B81J" +
       "ipopMak5bhrSuBrR\nojPjeespxm/htMsFqjQ5w0oBDG7WlCgxwtK7V795Zt" +
       "OWFw8LJzFgpSUByHO+wRxf5HJxhosY7oRd\nug0Dz7B4SD13ofnNL/FRsDJo" +
       "a8r7CFfGNe1lPWx6qGS66MkFWx+MMGAhLNUfuLb0rR9OnHcjT9GU\nEcpO9m" +
       "pGAivM65kYqUsf51wBsLQ7IVvs7N9f2nrm4reXV+XAS1F7QUwV7mQxscJpcE" +
       "OTSBRyTo79\nG39v/mO27NEP3MgLgQaphmIACMRti/OMvNjoyuQZposnhKpj" +
       "BYpXUTDQtE1h1vtZVytAAb6odwjI\nU9TN530PXvqk+jzXOJPNamxpb5hQER" +
       "uBnP9HDEJg/vKRwdnXrh/a5YFo1HXhc4p8uhVRZCkFW+7L\nDycmXpTB5rfT" +
       "XbWvrDU/5C6ulBMJi+KIQiD9YkXRpkk0THn+CdhyHU8xYAl/BFIVZL2wAoyE" +
       "rror\nCfAskgyCS+rnXu94+xJLFzo3TAMLKy4pSvGJdhfrVxUssu9mhuD6nN" +
       "oQ4FNCAX/H8O7+PYdXcM3t\n7NakP9LMUbIwEHtZ/chgIhF5+s+zx6qW5SRc" +
       "Y2fXwvvladuycVuGc07s7A4DNc+Xl3lNOfTUDf8L\n+NxukT3r8p2zSbUSDx" +
       "//kXQ86ZeKpIVKqulrFZIkStZUHn6uB45dWzLKt/IylosCz/DGjpVrqn+B\n" +
       "KJgnwAPpySECNVtl8rMzK3jk47uO/FaHUZzyBJKN2zwT8lduFlvpoC+ozvmb" +
       "uuzmgegz8gXl6Gnk\nAtSAiSqgdUHzp3M7/2WLAdbViQBl3fqikMzZ2VlBBw" +
       "05AQUqma6gr7a88/OZq0P1wn3imtFWUOnt\ne8RVgwtQrTMALS91Aqc+t3r5" +
       "qf1DVyLudJBsoKg8omkKwSqXuEcXCO2lyAN3Hzbsu32obQRqwyrg\n4E1qcp" +
       "RvG7BB3cOGmzhnG/tSa49nfVEJbRu0dWlfrCvmiyKxB1rqhpzEcOcs5SR+ul" +
       "l4QeIVU3jp\ntKf9hvvjhnZeBLwRbGbAnX+zLLw45t0HuaBVWbVWQ1tfQi1O" +
       "uIiihaK6wx0brtAKyYqfq9hOjOWl\nq7C079Ptx25+T69wjOVKGOPTlCpMdq" +
       "PYVl03XEwGfO8dT7hR+Riq5TdtrNJRrFismoyBymZPejKE\nFuSt5997xSWv" +
       "KxutTU6Q2451Fk973HppXsTyerk75UI6G5BCFLgp8JJVrIhMDLVOIWpcXOM4" +
       "3MZT\nWYu6xY6M4cXtiUkPSVlTCat3mTVRYmQtmH22wGKqwDfs+wkhIj9r/o" +
       "xxxxUOEnuZxMTJiFKbE1MY\nWfDYlQVaN7T+OwBaTT7QRotqg1k3kzH4s0UM" +
       "zsYTwtqsl7OWThVYmn1OsS5RwnQznE3qHpjuIOss\n1iVTt6cegau8rMo0L/" +
       "AGSmS2cZuSpdZymY21ldB2pD2z484yG7dbG0DZ5M/q4qrY7ZjDwv3QWkuc\n" +
       "WARU4sU6PxjmMmA4cjdgmP1/YJjjbGbvARiO3hUYXobCloC0WRwDLptGeTOQ" +
       "b6pyzzJ2JV9S8K8T\n8cCXVlzYs+qsHvha1JjMI7wcXsIxS1HsGdA29ukGic" +
       "lcwnKRD3X+cwKSBI9m9nFSF1q6/gOO5lRX\nEBIAAA==");
}
