package onedimjacobi;

import java.io.*;
import java.util.Arrays;

public class Sequential {
    final private static int MAX_ITERATIONS = 100000;
    
    private void run(int size) {
        double[][] u = new double[size + 2][size + 2];
        double[][] newu = new double[size + 2][size + 2];
        this.init(u, newu, size, size);
        double diff = 1.0;
        double valmx = 1.0;
        int iterations = 1;
        while (diff / valmx >= 1.0 * Math.pow(10, -5) && iterations <=
                 MAX_ITERATIONS) {
            diff = 0.0;
            valmx = 0.0;
            for (int i = 1; i < size + 1; i++) {
                for (int j = 1; j < size + 1; j++) {
                    newu[i][j] = (u[i - 1][j] + u[i + 1][j] + u[i][j - 1] +
                                    u[i][j + 1]) /
                                   4.0;
                    diff = Math.max(diff, Math.abs(newu[i][j] - u[i][j]));
                    valmx = Math.max(valmx, Math.abs(newu[i][j]));
                }
            }
            double[][] tmp = u;
            u = newu;
            newu = tmp;
            iterations++;
        }
        System.out.println("\nFinal result: ");
        this.printMatrix(u);
        if (iterations > MAX_ITERATIONS) {
            System.out.println(
              "[Master] Jacobi iterations did not converge in: " + iterations +
              " iterations.");
        } else {
            System.out.println("[Master] Jacobi converged after: " +
                               iterations + " iterations.");
        }
    }
    
    public static void main(String[] args) {
        int size = Integer.parseInt(args[0]);
        Sequential s = new Sequential();
        s.run(size);
    }
    
    private void init(double[][] u, double[][] newu, int rows, int size) {
        double sized = (double) size;
        for (int i = 1; i < rows + 2; i++) {
            double d = (double) i;
            u[i][0] = Math.sin(Math.PI * d / (sized + 1.0));
            newu[i][0] = Math.sin(Math.PI * d / (sized + 1.0));
            u[i][size + 1] = Math.sin(Math.PI * d / (sized + 1.0)) *
                               Math.exp(-1.0 * Math.PI);
            newu[i][size + 1] = Math.sin(Math.PI * d / (sized + 1.0)) *
                                  Math.exp(-1.0 * Math.PI);
        }
        for (int j = 0; j < size + 2; j++) {
            u[rows + 1][j] = 0.0;
            newu[rows + 1][j] = 0.0;
        }
    }
    
    private void printMatrix(double[][] u) {
        for (int i = 0; i < u.length; i++) {
            System.out.println(Arrays.toString(u[i]));
        }
    }
    
    public Sequential() { super(); }
    
    final public static String jlc$CompilerVersion$jl = "2.3.0";
    final public static long jlc$SourceLastModified$jl = 1238687026744L;
    final public static String jlc$ClassType$jl =
      ("H4sIAAAAAAAAAK1YaWwcRRauOTy+JsTjmBBiOyGJEYk2GQMSCDCXYxLiMImN" +
       "x4nABDntnppxOT3d\nna7qcSdEQEDKBBBIQMIlrh9ICJQfu4l2+cFqd6WFPT" +
       "iEFCTYP6AVIEDiEPyBCAHiVVX3dE/3eEwQ\nlrpcXfXq1Xv1vu+96jnxFWqh" +
       "FspSTCkx9LksO2BiKlpjZg6rjGbz28cVi+LCiKZQOgkT0+ren8nu\nP2Xu2R" +
       "ZHsSnUrRvDGlHo5Kxl2KXZyVlCHQtdYBragZJmMFdjRMdV636cf6u6vTeBlk" +
       "6hpUTPM4UR\ndcTQGXbYFEqXcXkGW3S4UMCFKZTRMS7ksUUUjRwEQUOHjSkp" +
       "6QqzLUwnMDW0ChfspraJLbGnN5hD\nadXQKbNslRkWZagrN6dUlEGbEW0wRy" +
       "gbyqFUkWCtQPejO1E8h1qKmlICweU5z4tBoXFwKx8H8Q4C\nZlpFRcXekuQ+" +
       "ohcYWh1eUfN44CYQgKWtZcxmjdpWSV2BAdQtTdIUvTSYZxbRSyDaYtiwC0Mr" +
       "F1QK\nQm2mou5TSniaoRVhuXE5BVLt4lj4EobODYsJTRCzlaGYBaI1lkr/dP" +
       "/49xdAxMHmAlY1bn8KFq0K\nLZrARWxhXcVy4Rk7e2z0VrsvjhAInxsSljLD" +
       "F76yK/f531dLmd4GMmMCi9Pqj5f39Z8e/qQ9wc1o\nMw1KOBTqPBdRHXdnhh" +
       "wT0L28ppFPZr3Jf0z869a7X8ZfxFHbKEqphmaX9VHUjvXCiNtvhX6O6FiO\n" +
       "jhWLFLNRlNTEUMoQ73AcRaJhfhwt0DcVNiv6jokQaoUnBs9dSP6184ahJXm8" +
       "38Y6AzRn6RxDAwxT\nRvdQS91j6LhAynOKasyQPXViDld7znwsBh71hdmlAR" +
       "S3GVoBW9Pqix+/cWjLTfcdlbHi+HINYui8\noPqsrx7FYkLvMo5CeUrDlqUc" +
       "4OxwDp/uf/LfyjNw5uA7JQexcC02n+QtLLq0afIY8ak3Cj0FkDGt\n9tz9+c" +
       "qn3n3p9ThKNEwgudrgVsMqKxrHgMeYbne78AxAZyAM4EZ7f33/jlPvvfnBeh" +
       "/KEIAIw6Ir\nOUPWhs/dMlRcgAzkq3/8h23fPNpy5Z/jKAm0g8TDFIALsHhV" +
       "eI86pgx5WYf7ksihzmLE8Q4GBzQf\ncJi3ad50SWxALHpCBoqEdebe1MXv/7" +
       "XzdeGxl9uWBpJgHjPJlIwf/0kLYxj/4InxR49/Vb0tAdw0\nTRlzhlKmPaMR" +
       "1YEl59WTi5tX4LD58uRQ10Ob6F9EiNtJuWwzZUbDkIwVTTPmcWGaiWyUCWQ+" +
       "kXDg\nJNIzkLggB05roEj6asYqAM8GqSG7oufYYxuefp8nD1MczHJOMmEpcs" +
       "TAQIy36yOT/L2fI7jHdxvo\nvk86kN6Qv3373qNrhedBdRvdF6em0OfCxqZc" +
       "2MqLjA8Vcui7wyffOZ6Oo/gUaiV0K9EVjR8W3SnB\n3yC9hlQc/NuuZ8+8zT" +
       "4UofXRxg3rc6J5YrcSIMIV71UyqT8+V46j1inUJUqkorPdimbzwE9BkaMj\n" +
       "7mAOLambry9YMjsP1djUF0Z6YNswzv38BH0uzfttEtpCZimcLD/jS+HpdNOo" +
       "+M8nM7zpdkTuktjl\nBmVHoTqXsNX90fMvfH+4ekWcZ66WCjcczqTLl9tp8z" +
       "vGkRPH+zuP/f8BgSAUO/I8V3qZ2HyVaNfI\n8CcYmM5DxFCraZGKwuAcUlTc" +
       "WxyGztkxfMv06OSWieHJ0bGdeRqN3bhFylB5Km5pfHjVC5+e+nii\nRyYieX" +
       "9YFynhwTXyDiEOp9Pk5FvTbAch/dof1py4c+LDGVlbu+vJukW3y5c99z+84f" +
       "q02qBaJOCW\nIxILijnR4xCkWucyDPkM8+gB5vUvdKEQplVv+TZ9RHnt9rhL" +
       "xRsYameGuUnDFazVuJUU6pKgbVNT\nbu0Q1yof3Yn8dRsu2tj5GeThBUpMxh" +
       "2cwHCH1Cc97Inao5x17Vkd8jVsT6bSe3NilvwnLlAviRK5\nLdYvGqqnR4dV" +
       "b6g4tN4aSfjdoheeJS5JloRIIktE4zBe0zhRxnyJzYun0glAjGXrYu56U8qP" +
       "MJSs\nGKQglt0WwEdQaXRkrOYVfy6CJ+d6lft1XsUFOHlzbVPXaPTWKy4+kq" +
       "EnEwPfxl9dPiBqeXJGoR5C\n6j8Xol8DdZd8YWNHzaPz4VndxCMhuIwF85T8" +
       "IojYzt8v4c2sE0Mm72gLnMTVkquQrjSsl+QNdTNv\nSn4Ri0txb/tl/vYjGl" +
       "wbefH25mS9JEa29kUGk9FyyN/HpX1iL97c2BRGi2KsAjlY5eY0OCVZhpzF\n" +
       "lewFUJahTgpMNkZmKYLMUhiZnG9b4NnlxnFXI2TyZrKhQV5e4+9Fly3e/wB7" +
       "UgXD9k4+EnvTi/uD\nzePOW6MW86ORmPPX/byhCwTRFCqO/g4BfIQ3d/DmUO" +
       "O9fKeeOBunqr/Vqerv4NQzAadqOjcH8lmz\nxfcCGolOWGM0Jmv6IkoXFajH" +
       "Ks+ijotV5+ywGqgD1cUdepHB7QQSFtuhQN5ymrCsGmFZ1YEk1eF/\nmPKPkh" +
       "WRn5LkDx7q2tN71//TzPxXpmfvR4nWHGor2poWvFgG+inTwkVpVau8ZkocnG" +
       "QoHfw65mOn\nTBnT2C+m/e92JxMAAA==");
}
