package schat.client;

import java.util.*;
import sessionj.runtime.*;
import sessionj.runtime.net.*;
import util.*;
import schat.common.*;
import schat.common.events.*;
import schat.client.gui.*;
import schat.client.net.*;

public class ChatClient
{
    final public static String
      DEFAULT_USER_NAME =
      "sj-user";
    final public static String
      DEFAULT_SERVER_ADDRESS =
      "localhost:8888";
    final private static Integer
      STATUS_TAB_KEY =
      new Integer(
      0);
    final private static Integer
      GLOBAL_COMM_KEY =
      new Integer(
      1);
    final private static String
      STATUS_TAB_TITLE =
      "Status";
    final private static String
      GLOBAL_COMM_TITLE =
      "Global";
    final private static String
      WELCOME_MESSAGE =
      "Welcome to the SJ Chat Client 0.1a by Ray (20090311).\n";
    final private static Integer
      NOT_CONNECTED_USER_ID =
      new Integer(
      -1);
    final private static String
      NOT_CONNECTED_USER_NAME =
      null;
    protected static boolean
      debug;
    private SJSessionParameters
      params;
    private UserList
      userList;
    private Map
      privateConversations;
    private ChatClientGui
      gui;
    private ChatClientCommunicator
      comm;
    private Integer
      userId =
      NOT_CONNECTED_USER_ID;
    private String
      userName =
      NOT_CONNECTED_USER_NAME;
    
    public ChatClient(String setups,
                      String transports) {
        super();
        this.
          params =
          TransportUtils.
            createSJSessionParameters(
            setups,
            transports);
        this.
          userList =
          new UserList(
            this);
        this.
          privateConversations =
          new HashMap(
            );
        this.
          gui =
          new ChatClientGui(
            this,
            userList);
        this.
          comm =
          new ChatClientCommunicator(
            this);
        gui.
          show();
        gui.
          createTab(
          STATUS_TAB_KEY,
          STATUS_TAB_TITLE);
        gui.
          appendMessage(
          STATUS_TAB_KEY,
          WELCOME_MESSAGE);
    }
    
    public boolean
      connect(
      String host_s,
      int port_s,
      String userName) {
        try {
            ChatClient.
              debugPrintln(
              "[ChatClient] Connecting to " +
              host_s +
              ":" +
              port_s);
            this.
              setUserId(
              comm.
                connect(
                host_s,
                port_s,
                userName));
            ChatClient.
              debugPrintln(
              "[ChatClient] Connected with user ID: " +
              this.
                getUserId());
            this.
              printStatusMessage(
              "Connected to " +
              host_s +
              ":" +
              port_s +
              " with user ID: " +
              this.
                getUserId());
            this.
              setUserName(
              userName);
            gui.
              createTab(
              GLOBAL_COMM_KEY,
              GLOBAL_COMM_TITLE);
            return true;
        }
        catch (SJIncompatibleSessionException ise) {
            this.
              reportError(
              "Client and server protocols incompatible.",
              ise.
                toString());
        }
        catch (SJIOException ioe) {
            this.
              reportError(
              "Connection failed.",
              ioe.
                toString());
        }
        return false;
    }
    
    public void
      sendMessage(
      Integer targetId,
      String msg) {
        Integer senderId =
          this.
            getUserId();
        ChatClient.
          debugPrintln(
          "[ChatClient] Preparing a message for sending: " +
          senderId +
          ", " +
          targetId +
          ", " +
          msg);
        if (targetId.
              equals(
              STATUS_TAB_KEY)) {
            gui.
              appendMessage(
              STATUS_TAB_KEY,
              this.
                formatStatusCommand(
                msg));
        } else
                  if (targetId.
                        equals(
                        GLOBAL_COMM_KEY)) {
                      comm.
                        sendEvent(
                        new MessageEvent(
                          senderId,
                          targetId,
                          msg));
                  } else {
                      synchronized (privateConversations)  {
                          if (privateConversations.
                                size() ==
                                0) {
                              System.
                                err.
                                println(
                                "[ChatClient] Shouldn\'t get here.");
                          }
                          for (Iterator i =
                                 privateConversations.
                                   keySet().
                                   iterator();
                               i.
                                 hasNext();
                               ) {
                              User user =
                                (User)
                                  i.
                                    next();
                              if (user.
                                    getId().
                                    equals(
                                    targetId)) {
                                  MessageEvent me =
                                    new MessageEvent(
                                    senderId,
                                    targetId,
                                    msg);
                                  Object ot =
                                    privateConversations.
                                      get(
                                      user);
                                  if (ot instanceof RequestorOutThread) {
                                      ((RequestorOutThread)
                                         ot).
                                        queueEvent(
                                        me);
                                  } else {
                                      ((AcceptorOutThread)
                                         ot).
                                        queueEvent(
                                        me);
                                  }
                                  gui.
                                    appendMessage(
                                    targetId,
                                    this.
                                      formatMessage(
                                      senderId,
                                      msg));
                                  break;
                              }
                              if (!i.
                                     hasNext()) {
                                  System.
                                    err.
                                    println(
                                    "[ChatClient] Shouldn\'t get in here: " +
                                    targetId +
                                    ", " +
                                    msg);
                              }
                          }
                      }
                  }
    }
    
    public void
      sendPrivateConversationRequest(
      Integer targetId) {
        Integer requestorId =
          this.
            getUserId();
        if (targetId.
              equals(
              requestorId)) {
            this.
              reportError(
              "Invalid private conversation request.",
              "Cannot initiate a private conversation with yourself.");
        } else {
            ChatClient.
              debugPrintln(
              "[ChatClient] Preparing a private conversation request: " +
              requestorId +
              ", " +
              targetId);
            comm.
              sendEvent(
              new PrivateConversationEvent(
                requestorId,
                targetId));
        }
    }
    
    public boolean
      handleEvent(
      ChatEvent e) {
        ChatClient.
          debugPrintln(
          "[ChatClient] Handling event: " +
          e);
        if (e instanceof UserJoinedEvent) {
            UserJoinedEvent uje =
              (UserJoinedEvent)
                e;
            userList.
              addUser(
              new User(
                uje.
                  getUserId(),
                uje.
                  getUserName()));
        } else
                  if (e instanceof MessageEvent) {
                      MessageEvent me =
                        (MessageEvent)
                          e;
                      Integer senderId =
                        me.
                          getSenderId();
                      Integer targetId =
                        me.
                          getTargetId();
                      String msg =
                        me.
                          getMessage();
                      if (targetId.
                            equals(
                            GLOBAL_COMM_KEY)) {
                          gui.
                            appendMessage(
                            GLOBAL_COMM_KEY,
                            this.
                              formatMessage(
                              senderId,
                              msg));
                      } else
                                if (targetId.
                                      equals(
                                      this.
                                        getUserId())) {
                                    gui.
                                      appendMessage(
                                      senderId,
                                      this.
                                        formatMessage(
                                        senderId,
                                        msg));
                                } else {
                                    System.
                                      err.
                                      println(
                                      "[ChatClient] Shouldn\'t get in here 3: " +
                                      targetId +
                                      ", " +
                                      msg);
                                    return false;
                                }
                  } else
                            if (e instanceof PrivateConversationEvent) {
                                PrivateConversationEvent pce =
                                  (PrivateConversationEvent)
                                    e;
                                Integer requestorId =
                                  pce.
                                    getRequestorId();
                                Integer targetId =
                                  pce.
                                    getTargetId();
                                if (!targetId.
                                       equals(
                                       this.
                                         getUserId())) {
                                    System.
                                      err.
                                      println(
                                      "[ChatClient] Shouldn\'t get in here: " +
                                      targetId);
                                    return false;
                                }
                                User requestor =
                                  userList.
                                    findUser(
                                    requestorId);
                                synchronized (privateConversations)  {
                                    if (privateConversations.
                                          keySet().
                                          contains(
                                          requestor)) {
                                        ChatClient.
                                          debugPrintln(
                                          ("[ChatClient] Duplicate private conversation request rejected" +
                                           ": ") +
                                          requestor);
                                        return false;
                                    }
                                }
                            } else
                                      if (e instanceof UserLeftEvent) {
                                          userList.
                                            removeUser(
                                            ((UserLeftEvent)
                                               e).
                                              getUserId());
                                      } else
                                                if (e instanceof QuitEvent) {
                                                    
                                                } else {
                                                    System.
                                                      err.
                                                      println(
                                                      "[ChatClient] Shouldn\'t get in here: " +
                                                      e);
                                                    return false;
                                                }
        return true;
    }
    
    public void
      openPrivateConversation(
      Integer key,
      Object ot) {
        User user =
          userList.
            findUser(
            key);
        ChatClient.
          debugPrintln(
          "[ChatClient] Opening private conversation request with: " +
          user);
        this.
          printStatusMessage(
          "Private conversation opened with: " +
          user);
        gui.
          createTab(
          key,
          user.
            getName());
        synchronized (privateConversations)  {
            privateConversations.
              put(
              user,
              ot);
        }
    }
    
    public void
      closePrivateConversation(
      User user) {
        synchronized (privateConversations)  {
            Object ot =
              privateConversations.
                get(
                user);
            if (ot instanceof RequestorOutThread) {
                ((RequestorOutThread)
                   ot).
                  disconnect();
            } else {
                ((AcceptorOutThread)
                   ot).
                  disconnect();
            }
            privateConversations.
              remove(
              user);
            gui.
              disableTab(
              user.
                getId());
        }
        this.
          printStatusMessage(
          "Closed private conversation with: " +
          user);
    }
    
    public void
      closePrivateConversation(
      Integer key) {
        synchronized (privateConversations)  {
            for (Iterator i =
                   privateConversations.
                     keySet().
                     iterator();
                 i.
                   hasNext();
                 ) {
                User user =
                  (User)
                    i.
                      next();
                if (user.
                      getId().
                      equals(
                      key)) {
                    this.
                      closePrivateConversation(
                      user);
                    break;
                }
                if (!i.
                       hasNext()) {
                    System.
                      err.
                      println(
                      "[ChatClient] Shouldn\'t get in here: " +
                      key);
                }
            }
        }
    }
    
    public void
      disconnect(
      ) {
        gui.
          disableTab(
          GLOBAL_COMM_KEY);
        gui.
          performDisconnect();
        if (comm.
              isConnected()) {
            ChatClient.
              debugPrintln(
              "[ChatClient] Sending user left event...");
            comm.
              sendEvent(
              new UserLeftEvent(
                this.
                  getUserId()));
            synchronized (privateConversations)  {
                for (Iterator i =
                       privateConversations.
                         keySet().
                         iterator();
                     i.
                       hasNext();
                     ) {
                    this.
                      closePrivateConversation(
                      (User)
                        i.
                          next());
                }
            }
            userList.
              removeAll();
        }
        this.
          setUserId(
          NOT_CONNECTED_USER_ID);
        this.
          setUserName(
          NOT_CONNECTED_USER_NAME);
        if (comm.
              isConnected()) {
            comm.
              disconnect();
        }
        this.
          printStatusMessage(
          "Disconnected from server.");
    }
    
    private String
      formatStatusCommand(
      String msg) {
        return Utils.
                 makeTimeStamp() +
        " <User> " +
        msg;
    }
    
    private String
      formatMessage(
      Integer senderId,
      String msg) {
        return Utils.
                 makeTimeStamp() +
        " <" +
        userList.
          findUser(
          senderId).
          getName() +
        "> " +
        msg;
    }
    
    public static void
      main(
      String[] args)
          throws Exception {
        boolean debug =
          Boolean.
            parseBoolean(
            args[0]);
        String setups =
          args[1];
        String transports =
          args[2];
        ChatClient.
          debug =
          debug;
        ChatClient c =
          new ChatClient(
          setups,
          transports);
    }
    
    public void
      printStatusMessage(
      String msg) {
        gui.
          appendMessage(
          STATUS_TAB_KEY,
          Utils.
            makeTimeStamp() +
          " " +
          msg);
    }
    
    public void
      reportError(
      String title,
      String msg) {
        gui.
          showMessageDialog(
          title,
          msg);
    }
    
    public Integer
      getUserId(
      ) {
        return userId;
    }
    
    public void
      setUserId(
      Integer userId) {
        this.
          userId =
          userId;
    }
    
    public String
      getUserName(
      ) {
        return userName;
    }
    
    private void
      setUserName(
      String userName) {
        this.
          userName =
          userName;
    }
    
    public static void
      debugPrintln(
      String m) {
        if (ChatClient.
              getDebug()) {
            System.
              out.
              println(
              m);
        }
    }
    
    public static boolean
      getDebug(
      ) {
        return debug;
    }
    
    public SJSessionParameters
      getSessionParameters(
      ) {
        return params;
    }
    
    final public static String
      jlc$CompilerVersion$jl =
      "2.3.0";
    final public static long
      jlc$SourceLastModified$jl =
      1238676054775L;
    final public static String
      jlc$ClassType$jl =
      ("H4sIAAAAAAAAALVaa2wcVxW+s+vYjuM81nGclx1vUreNcbJO0hKlNWnrOuvE" +
       "6dpxveuUmFSb8ez1\netzZmenMXXudRlFCKAmRoIKmFKpCflCBaCsVUgGqQA" +
       "WJlgIBIQUp5U8qVUUQCYrIH4iqEnHuY/Yx\nM/uw20S909n7Ot937jnnnnvH" +
       "r36IltgWitjYtlVDn4mQeRPb7GlMzmCF2JH4wVHZsnFqQJNtOwEN\nSeXYLf" +
       "Xwj0NfPBBA0gRq0Y1+TZXtxLRlZNPTiWnVzlkobBrafFoziJjRM8f9d3w894" +
       "ezBzcG0coJ\ntFLV40QmqjJg6ATnyARqzuDMJLbs/lQKpyZQSMc4FceWKmvq" +
       "ceho6CDYVtO6TLIWtsewbWiztGOL\nnTWxxWQ6lTHUrBi6TaysQgzLJmhVbE" +
       "aelXuzRNV6Y6pN+mKofkrFWsp+Ep1EgRhaMqXJaejYFnNY\n9LIZewdpPXRv" +
       "UgGmNSUr2BlS94SqpwjqdI/IM+56BDrA0IYMJtNGXlSdLkMFauGQNFlP98aJ" +
       "pepp\n6LrEyIIUgjaUnRQ6NZqy8oScxkmC1rn7jfIm6LWUqYUOIWiNuxubCd" +
       "Zsg2vNilbrUH3z/86P/jcM\nKw6YU1jRKP56GLTJNWgMT2EL6wrmA29mIxeG" +
       "jmTbAwhB5zWuzrxP/50/G49d/2Un77PRp88hZotJ\n5ePd7R1X+v+6NEhhNJ" +
       "qGrVJTKGHOVnVUtPTlTLDutvyMtDHiNP5q7DdHTr2M/xFAjUOoXjG0bEYf\n" +
       "QkuxnhoQ7w3wHlN1zGsPTU3ZmAyhOo1V1RvsN6hjStUwVccSeDdlMs3ecyZC" +
       "qAGKBP+9hvi/Vvog\naPnAtEwGNBXrJGLPENRFsE3so7alHLUVaDqqsLajJd" +
       "1ydNoVc5IEjNrd3qWBKR4wtBS2ksoPPvj9\niegjXznH14ralwBE0Fo2fYRP" +
       "HylMjySJzbuaWiHXUr9lyfPUO3Knr3R8+x35O6Bz4G6rxzGjJs3V\n0ScM2l" +
       "UxeAwUXG8I3mSwjKTSeur6hhf+/MO3AyjoG0Bi+cpBw8rIGrUBx2NahDh3C5" +
       "hOl9uA/WT/\n6/zw61cvX9taMGVYAI+HeUdSD9ni1rtlKDgFEagw/fMfHfj3" +
       "s0vu+0kA1YHbQeAhMpgLePEmt4wS\nT+lzog7lEoyhZVMe4k0EFDRXRJg+m9" +
       "n7SliKRijgQWi/sLV76YM2huijhZsPLFeriwOLaTfP1O94\n9xfL3mZKccLf" +
       "yqI4GceEO1OoYCIJC2Oov/at0Wef+/DsF4LgvqbJzYKgejM7qalKDoasLfU/" +
       "yiBF\nLeufl/pWfW27/VNmBUvVTCZL5EkNQ7yWNc2Yw6kkYQErVBQcWUwCZT" +
       "VPQmyDMJnUYCKuDlOaBQv2\niR6Rda0Xvtn94rs0vphMX21UVwxpgP1eDZsC" +
       "40VlRHgApvU9OdZ8p0Sf3fmhiA1FTKUdiOq0oBSI\nF09wes3d8ccPHju3he" +
       "mFC2NzFk3c66nO5YWEWFsIZt9W0cEG6c5VsD/1xH9OX/rTc80BFJhADao9\n" +
       "qOqyRtVrj3CP8onZrimOvzn+3Zt/JO8xYyiYMAXWnvMGn8NykXftuTobqv/R" +
       "xUwANUygVWzflXVy\nWNay1FQmYOe0B0RlDC0vaS/dBXnI78u7aLvbfYrEup" +
       "2nEPTgnfam740uf1kD5X4obcJf2lz+IoER\nN9gz27M2p76Hjexkzy18PYME" +
       "5FL90rousHmbJTE5MNl90cH+8VgiOR6PjiVH+oej+TVeISE2xz0l\nSAahrB" +
       "VI1vogWaEZiqxNGza5fw/8ow0P+QCi73sZGvp4AJC0OUgAyGHA0r9v31g0Hq" +
       "8AhyriASjr\nBZz1HjgmfRksJx7UZlrqrEywg2FFPNGfGI8nE/0PJx+JHnFc" +
       "LlRwuSHIp9Jcz/6AHoSyQQDa4A9o\npJI+DjpYVu6PHXq4P5YcODQ8TMHQ+l" +
       "h5ufdBaRdy233WpZ7mrVk2JF6T/FVFukgMJWKVDKNNmGiH\nANDhB2C/Zkxy" +
       "CzxSE4BQsQJqQaBC6RQIOn0Q7H4Ma4qRwWFihMk0DscPhmlmEeapRXhHZKcc" +
       "npwP\nj8nz4a27duy4b8c9O3d2R5roBMnaluyxaAzQRpPDYLj9+6vhpa60We" +
       "Dd7G8quCa5a0YOJUBNIyPR\ngUR0H3floX1VDOZzULYI6Vv8pc/UJH2tj/Qq" +
       "gaQFShhKl5Df5S/f8MoPELTUtAwCIRenHARLUngy\nm7a9+8WopWYghZ4VOf" +
       "7XN730t9c/GGvlGRU/CN3hOYsUj+GHIRaQl5k0RdhcSQLr/VbP5ldPjr03\n" +
       "yQ8JLaUpRVTPZj578S+4+6FmxSftbZg0DA3LOtMa5KylWlsqNNUttNbtr7UT" +
       "Xq2xtOBgjiY7siVn\nbCewbc1v2FZWJ2oGR3RMYKuO8+pR2hnD0c7mgNzLSA" +
       "FthNIjAPX4AzpDHycJaqQ7FE04HPHrS7L8\ndFaNjIse5eVRU90m5G3zl3dO" +
       "yGsV0R328VngwE7jeerLC5nQsGxW5hcR8iL+8r4q5AWBgTP9Jg+1\nwiFmf1" +
       "YtL+9OKL1CXq+/vG8IeXUQzDKOwLtLBNJlLAgcgH5ZXVVkOCWUl7wDyk4hea" +
       "e/5OeF5Hq6\nkkOpQoTxnfAeKLvEhLv8J3yh2DRGhCv0iCmZ223miS0b212U" +
       "iyLqjR3lLgKYJ579/I3mL8tvPR4Q\nU81D7CCGuV3Ds1jLp6+tbLpWmG17xf" +
       "R1mF2HFBLIYPzB7ru3Lfs7nJ/KHA1DonIMk6ylJ5z0jp0Z\n5QWfGTtdXN14" +
       "QrMbHw1Oq78NsMSS56KeW57SQX2lGWiTVQqUKW1jybltAMqQWNIh15KyVSta" +
       "MtdZ\nJFjYP3pYZdbkZwgIS0FVZy5fwzHm+xAlIRvXYVXozznWda7IMIJ5E3" +
       "q5aMpKbd8robgbSlpQTC+M\nYqBwSorVSOcSQctsrKeGYd3lNHYrpm7WUFNs" +
       "+Bvuc1jMw85TXUpsL5SLgtjFhRGTCrtIrDqnN2n4\nA06j3vA7hp/MYh7e3/" +
       "DQKp7fW1NKZisU57LqtUWRcQJnhwicECMNPQKhQQd3p7EzSl+rs30HVnBa\n" +
       "1lMaZgP8jZJVXPZQu+xHjSZmtwS1W4s3QJ/7An5UrU7pCiR1hol1nwX0X7mC" +
       "5V31N8irfkRhw5FW\ncJ78/4tew1DJGtIkojrJawStUzTDxjWzZBXvexbxfT" +
       "9usN9Jdwlud91OZ7u+OBo1uhkcqaW9gsbe\nhdFwQFZlcIOgppRqFwV1L2aU" +
       "8/4oIKXJBuRK0jGB9Fg5pCerKbyGiP0RQavZnSfhJ2qaXIH7s9F8\nDrdmez" +
       "y69sRnyqAPGrFggBfGYOEbjxSEFJjTEFuPP4GFbzW03A0UZgSVGT8qPgc7+i" +
       "4ugyq6vPfz\nD/sCwE94l4JdNwI/b+til9p1k7LtpFyl3828n8VKvnYxjE15" +
       "RvReqbMKox7f0HQv1XRrTkI02ZXa\nyrDm92D0dKZhPc0/y9D8RFqdc6+wE/" +
       "BWF4L6gGbomF5HO238jlc1IvnPkNCY88X3EsfHZH1yT5bC\ncBRXKBzaeDVX" +
       "eR29TKI5BZs0brHJmiH7ychqhSDm6MdbUxrE4FgqzYq1m110LK7Bp7YTOO9b" +
       "YFs8\nMhQ5VpkoXENkcDaTpwSBpxZGoCgy9NTIYjckNBY2DYtELYsfGMvs+J" +
       "W/EPgQgcO0dEoQOXV7thPp\nQTjlpTEZL5xQ+fZW825CgcKWLT0tgD69aJOp" +
       "vn1LBwCtXYz2k+3X7dB4XuA+f5sU/CjYh1Bw/sTu\n2Teqb9ifAYjPCKjPlI" +
       "P6KWzY0gQ7YpXgXbw7rhTILwjkF/yQL353q5GSQlAzu/McpdFGq5To1chp\n" +
       "MzS+KDi9+ClxqtmidIIawaL2UUa03Xt+qu6xNEa+Igi8cpssfx42V8Dpezt6" +
       "mo07XR4z7O9NhSs5\n+oV6nedPj/gfyChbrhzb+msz9DuexTh/xNIQQ41TWU" +
       "0r/mZY9F5vWnhKZVwb+BdEk4H+EthK8d0g\npBn8haKTzvBOZ2HnZp3oj3Ps" +
       "au5qDv0fdNG+N3UlAAA=");
}
