package schat.client.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import schat.common.*;
import schat.client.*;

public class ChatClientGui implements WindowListener {
    final private static String GUI_TITLE = "SJChat Client";
    final private static int MAIN_PANE_WIDTH = 600;
    final private static int USER_LIST_WIDTH = 100;
    final private static int APP_HEIGHT = 400;
    final private static int MESSAGE_DIALOG_MAX_LINE_LENGTH = 100;
    private ChatClient client;
    private JFrame topLevelFrame;
    private MainPane mainPane;
    private ChatMenuBar menuBar;
    private ChatToolBar toolBar;
    private UserList userList;
    private JScrollPane userListScroller;
    
    public ChatClientGui(ChatClient client, UserList userList) {
        super();
        this.client = client;
        this.topLevelFrame = new JFrame(GUI_TITLE);
        this.mainPane = new MainPane(client);
        this.menuBar = new ChatMenuBar();
        this.toolBar = new ChatToolBar(client, this);
        this.userList = userList;
        this.userListScroller = new JScrollPane(userList);
    }
    
    public void showMessageDialog(String title, String msg) {
        String m = "";
        for (int len = msg.length(); len > MESSAGE_DIALOG_MAX_LINE_LENGTH;
             len = msg.length()) {
            int index = msg.indexOf('\n');
            if (index >= 0 && index < MESSAGE_DIALOG_MAX_LINE_LENGTH) {
                index += 1;
                m += msg.substring(0, index);
            } else {
                index = MESSAGE_DIALOG_MAX_LINE_LENGTH;
                m += msg.substring(0, index) + "\n";
            }
            msg = msg.substring(index);
        }
        m += msg;
        JOptionPane.showMessageDialog(topLevelFrame, m, title,
                                      JOptionPane.ERROR_MESSAGE);
    }
    
    public void createTab(Integer key, String title) {
        synchronized (mainPane)  {
            {
                mainPane.addChatTab(key, title);
                mainPane.setSelectedIndex(mainPane.getTabCount() - 1);
            }
        }
    }
    
    public void disableTab(Integer key) {
        synchronized (mainPane)  { mainPane.disableChatTab(key); }
    }
    
    public void closeTab(Integer key) {
        synchronized (mainPane)  { mainPane.removeChatTab(key); }
    }
    
    public void appendMessage(Integer key, String msg) {
        mainPane.appendMessage(key, msg);
    }
    
    private void assemble() {
        mainPane.setPreferredSize(new Dimension(MAIN_PANE_WIDTH, APP_HEIGHT));
        userList.setPreferredSize(new Dimension(USER_LIST_WIDTH, APP_HEIGHT));
        topLevelFrame.setJMenuBar(menuBar);
        topLevelFrame.getContentPane().add(toolBar, BorderLayout.PAGE_START);
        topLevelFrame.getContentPane().add(mainPane, BorderLayout.CENTER);
        topLevelFrame.getContentPane().add(userListScroller, BorderLayout.EAST);
        topLevelFrame.addWindowListener(this);
        topLevelFrame.setDefaultCloseOperation(
          WindowConstants.DISPOSE_ON_CLOSE);
        mainPane.requestFocusInWindow();
        this.addAnonymousListeners();
        topLevelFrame.pack();
        topLevelFrame.setVisible(true);
    }
    
    public void performDisconnect() { toolBar.performDisconnect(); }
    
    private void addAnonymousListeners() {
        topLevelFrame.addWindowFocusListener(
          new WindowFocusListener() {
              
              public void windowGainedFocus(WindowEvent e) {
                  mainPane.requestFocusInWindow();
              }
              
              public void windowLostFocus(WindowEvent e) {  }
          });
    }
    
    public void windowClosing(WindowEvent e) {
        try {
            client.disconnect();
        }
        catch (Exception x) {
            ChatClient.debugPrintln("[ChatClientGui] Exception: " +
                                    x.getMessage());
        }
    }
    
    public void show() {
        SwingUtilities.invokeLater(
          new Runnable() {
              
              public void run() {
                  try {
                      ChatClientGui.this.assemble();
                  }
                  catch (Exception x) {
                      System.err.print("[ChatClientGui] ");
                      x.printStackTrace();
                  }
              }
          });
    }
    
    public void windowActivated(WindowEvent e) {  }
    
    public void windowClosed(WindowEvent e) {  }
    
    public void windowDeactivated(WindowEvent e) {  }
    
    public void windowDeiconified(WindowEvent e) {  }
    
    public void windowIconified(WindowEvent e) {  }
    
    public void windowOpened(WindowEvent e) {  }
    
    final public static String jlc$CompilerVersion$jl = "2.3.0";
    final public static long jlc$SourceLastModified$jl = 1238676054889L;
    final public static String jlc$ClassType$jl =
      ("H4sIAAAAAAAAALVZa2xUxxWeXdtrGzbgNQbz8AM70OI6LIEqFYl/wNYs9pI1" +
       "3ngXAq6j5frurH3x\n3Xtv78zaC4nSkCohSaW+Am2pWlKRSFUqfrRELT9atZ" +
       "UKfUeVqET6I0mbUtGoNBVVlYZWtOqZx77u\n3V2DayPNMJ45c853zpxz5szd" +
       "8++hBmKjIMGEaKZxNEiPWZjw3pw8ilVKgvF9McUmODWoK4QkYCGp\nHvmvdv" +
       "A7gaeHvcgzjloNM6RrCklM22Z2ajoxrZGcjTZapn5sSjep5Oji8dC9t+d+fX" +
       "Lfhjq0chyt\n1Iw4VaimDpoGxTk6jvwZnJnENgmlUjg1jgIGxqk4tjVF144D" +
       "oWmAYKJNGQrN2piMYWLqs4ywlWQt\nbHOZ+cko8qumQaidValpE4paokeVWW" +
       "Vblmr6tqhG6EAU+dIa1lPkk+hJ5I2ihrSuTAHhmmhei22c\n47a9bB7Il2kA" +
       "004rKs5vqZ/RjBRF3c4dBY03PQwEsLUxg+m0WRBVbygwgVoFJF0xprbFqa0Z" +
       "U0Da\nYGZBCkXrqzIFoiZLUWeUKZykaK2TLiaWgKqZm4VtoWi1k4xzgjNb7z" +
       "izktMa9fn/80Lsg41w4oA5\nhVWd4ffBpi7HpjGcxjY2VCw23soGT0UOZzu8" +
       "CAHxagexoAltvngg+u6PugXNhgo0o9wXk+rtj3V0\nXgn9qbmOwWiyTKIxVy" +
       "jTnJ9qTK4M5Czw7jUFjmwxmF/88dhPDz/1LXzDi5oiyKeaejZjRFAzNlKD\n" +
       "ctwI46hmYDE7mk4TTCOoXudTPpP/DeZIazpm5miAsaXQaT7OWQihRmgeaP9C" +
       "4t9K1oEDDk4rdFDX\nsEGHslqQHKWon2JCyQSx1QmiwuqEypcnprLahJM6xw" +
       "SsmPN4QLcOZ5zp4JTDpp7CdlL95rVfPhF+\n+PnnxKkxT5PQKOriUoJCShCk" +
       "BMukII+Hs1/F3FKYLWTbyjEWLrkTVzrP/Ez5OhwCGINoxzHX1TNX\nz3rYtK" +
       "NmNhksxmIERgq4SlJte+rd9V/97auXvaiuYkaJFib3mnZG0ZlT5EOoVYpzro" +
       "AvbXJ6dCXZ\nf3th5LWrv3prS9G3KdrkCjn3ThYyvU7z26aKU5CSiuy//O/h" +
       "my82PPhdL6qHOIRMRBXwHwjrLqeM\nstAZyKchpktdFC1PuxRfRsFAcyUKs9" +
       "7Px8zRmqBBSKH7pfNtYh1bDLCuVXgRHFebQwee5G592nf/\nGz9YfpkbJZ8P" +
       "V5YkzjimIroCRRdJ2BjD/Ftfib14+r2Tn6iDeLYs4RYU+azspK6pOdjSXh6Q" +
       "TIMU\n86y/Xhho+exW8j3uBc1aJpOlyqSOIYErum7O4VSS8gwWKMmWPEmBsf" +
       "yTkOwgbyZ1YCTMYXlmwYMr\npJPg2rZTX+r72hss4VjcXmuYrThSL/97FUXt" +
       "ZRFSjI78+jpXBB0g2OYBwkk2e1jfV2CPOHvEzd6J\nmN2LhoMkMyNM4O+LP7" +
       "bvyHO93HYCENvQz7qtgvF213SuIMTP1/zA/b6aQbiXXXdFH9We+OeJC785\n" +
       "7fci7zhq1MhezVB0dgRkv4i6ConeweL4Dw+cvfU6fZs7TNHNGbCOnDtPHVRK" +
       "InDn1dmA79svZbyo\ncRy18MtaMehBRc8ydxqH65YMyskouqdsvfzqFPfEQC" +
       "GMO5whViLWGWDF/AhjRs3GTY6YYjZ+EFpA\nxlTAEVMecPR7IMeBZ2wU7sLm" +
       "H+L7u3nfK061joJ0ZmWKGi1bm1UoqOIjvAjKUdQ8dCCSTEQS0XDe\n3Vq4tz" +
       "Atg6JA4OHrQXz9gTKALOBXSYCrXAALIct5RaCQmcJ26x+/8coHJ07u9LKc3j" +
       "DLLAuHViJz\nf5aVY8+eP925/NQfPsMDB3zwEGMarqAdG+9i3W7WhUCjlSOh" +
       "yP5kLLQ/nHw0sicxTNweFbO1DNzM\ns7J0+ELXK9dfuzbWJvKyqK/udZU4pX" +
       "tEjcWPbLnFEk1PLQmc+lJ/z/knx96eFLVHa3liChvZzAMv\n/Q737farFe7Q" +
       "Ok0c7wq49SqfQps8hTb3KbDBEMym2CBxZyY8EA+PJaOReEKYkM2PVgfQD221" +
       "BLC6\nKgDP59lg/I4ALAvFYsnhcGRoODGP7F3Q2qXs9nmUP3JHsrtGwvF4aC" +
       "ic3BMJRUeHkiOhQ2AK8KZo\neP9QTVs0Q9sAbZ3Es86Fx2KDtBsGz+C7QbhP" +
       "LcRyvxDijDsmpEsKQvn/3UJmWDcNOYKaVhTPYn2v\nDR6Vj/FWFm+5IJmD8A" +
       "7u40vVpTEJnVJaZ2VplpTWlIFcHFOMgiD33TUiKarL65Qaovz/bnlZKQ/e\n" +
       "OEb24/Dik+I6KhabI4KotsRuKbG7ssTjeYnUNPV5JSYEUW2b9kiJPZUlfipv" +
       "06y87fkVXJ3lh6D1\nSpa9lVk+LVm25FnGVdvUdVzQpr3ML8RqyWHxhNcjig" +
       "7Ovq+kTvDkmXTyZK7M0SD4HRjlUch+5hwT\nh+GeZtmys9r7j2fKk4f+7n9W" +
       "ufSYVwp8BG4p8OKtOnPjQgHSwoW2ALetNQuQEf4KLpYAdfFdfR++\nb/mfoU" +
       "qu8gAIyMkxDE9+I5G/oPnLQLnrl0G3Q1cnnsDshkfqprWfe3lpIKoJ1+O+fN" +
       "NAeQ2xzC4H\nyo22oaw63wlth/SNHQ7f4GdbcrCOatJbLAIHeTd/0flFKJzJ" +
       "tDk3AkZRpvAeTdFNUf2MWGJ3jKL6\nWVNLcSZnnMVmqZwq058rU287tMNSvc" +
       "MLUi/vu+6C5Q51PgtOqtoYqquEMsm1qqzbucq6nauqG3tM\nzUjdZu5ON0/x" +
       "bjk3vwavwrWb0gh7B1VVoZSXe6YcOMtBRAInSwn8AuRIVTfJ4sD+KLTHJezH" +
       "Fxwq\npedZC/v34YZWLAsbKRksi+U67EZYC+0Zqcoz1VSZrolxXgUugfEhee" +
       "PMpPg0dQY5saOc+49yk2+E\ndlbiPHt3Jr9jnK9DbFvYZt829mgEXnYG3A+V" +
       "jV0bMDPsZmgvS8AvL5Fhr1K0WkmlQoZpHMuYWZK/\nP8lCQDMr90G7KEFfXF" +
       "A85pPk+ooXfJiN59fr9+Dxc3zDIMSsfF5WCdlrrpC9Vkkz9vS8LDW7vET+\n" +
       "8xe4sdidtlDrszfSmxLjmwuyfon6tYD+A55vwr4hlfLnfur/t/AWaO9I9O8s" +
       "JfrbFPmL3rEY0LdC\nuy6hX19C6J4GyDAC+h6sLJ7pGf4bEv+NpcS/ogS/Bg" +
       "lSS2uLgZ85/k2J/+ZS4m8vOH5k8dAzx39f\non9/KdFvLDj+KFQDdw09B1m1" +
       "7HcV9sF8reunUfEDntp75ciWn1iBX/BfCgo/sjVGUVM6q+ulnydL\nxj7Lxm" +
       "mNa90oPlZaHHk/vCedj2CK6qBnMD0fEVRB0K+UqvCpo5RoO0UNnIj9scPK3z" +
       "clXwfFd9cc\n+h/Aa6WrTx4AAA==");
}
