package schat.client.gui;

import java.awt.BorderLayout;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import schat.common.*;
import schat.client.*;

public class ChatTab extends JPanel implements ActionListener {
    final private static String SEND_ACTION = "SEND_MESSAGE";
    private MainPane mainPane;
    private Integer key;
    private JTextArea view;
    private JScrollPane viewPane;
    private JTextField input;
    
    public ChatTab(MainPane mainPane, Integer key) {
        super(new BorderLayout());
        this.mainPane = mainPane;
        this.key = key;
        this.view = new JTextArea();
        this.view.setEditable(false);
        this.view.setLineWrap(true);
        viewPane = new JScrollPane(view);
        this.input = new JTextField();
        this.input.setActionCommand(SEND_ACTION);
        this.input.addActionListener(this);
        this.add(viewPane, BorderLayout.CENTER);
        this.add(input, BorderLayout.PAGE_END);
        this.addAnonymousListeners();
        this.requestFocusOnInput();
    }
    
    private void requestFocusOnInput() {
        SwingUtilities.invokeLater(new Runnable() {
                                       
                                       public void run() {
                                           try {
                                               input.requestFocusInWindow();
                                           }
                                           catch (Exception x) {
                                               System.err.print("[ChatTab] ");
                                               x.printStackTrace();
                                           }
                                       }
                                   });
    }
    
    public void actionPerformed(ActionEvent e) {
        String action = e.getActionCommand();
        if (action.equals(SEND_ACTION)) {
            String msg = this.getAndClearInput();
            ChatClient.debugPrintln("[ChatTab] Message written for " +
                                    this.getTabKey() + ": " + msg);
            this.mainPane.getClient().sendMessage(this.getTabKey(), msg);
        } else {
            System.err.println("[ChatToolBar] Shouldn\'t get in here: " +
                               action);
        }
    }
    
    public void enableInput() {
        input.setEnabled(true);
        this.requestFocusOnInput();
    }
    
    public void disableInput() { input.setEnabled(false); }
    
    public void appendText(String text) {
        {
            view.setText(view.getText() + text);
            { this.moveCaretToEnd(); }
        }
    }
    
    private void moveCaretToEnd() {
        SwingUtilities.invokeLater(
          new Runnable() {
              
              public void run() {
                  view.setCaretPosition(view.getDocument().getLength());
              }
          });
    }
    
    private String getAndClearInput() {
        synchronized (input)  {
            String m = input.getText();
            input.setText("");
            return m;
        }
    }
    
    public Integer getTabKey() { return key; }
    
    private void addAnonymousListeners() {
        this.addFocusListener(new FocusListener() {
                                  
                                  public void focusGained(FocusEvent e) {
                                      input.requestFocusInWindow();
                                  }
                                  
                                  public void focusLost(FocusEvent e) {  }
                              });
        this.addMouseListener(new MouseListener() {
                                  
                                  public void mouseClicked(MouseEvent e) {
                                      input.requestFocusInWindow();
                                  }
                                  
                                  public void mouseEntered(MouseEvent e) {  }
                                  
                                  public void mouseExited(MouseEvent e) {  }
                                  
                                  public void mousePressed(MouseEvent e) {  }
                                  
                                  public void mouseReleased(MouseEvent e) {  }
                              });
    }
    
    final public static String jlc$CompilerVersion$jl = "2.3.0";
    final public static long jlc$SourceLastModified$jl = 1238676054870L;
    final public static String jlc$ClassType$jl =
      ("H4sIAAAAAAAAALVYa2xcRxWe3bV37XgbZ+0kDX4kjZOIWqnXFKlViH/Q1WI3" +
       "dtax8W4DdV0543tn\ndye5e+/Nnbm761BVbZFwKRISakILolSVqlSgSkArHj" +
       "8QIPqAvlQplVr+tD8ogkpQBH8gQgVxZu7c\nfdy7dhqQV5rZ2Zkz5zXfOXNm" +
       "n/0QdTMHpRlhjFrm2TRftwmTvbV2lmicpfNzi9hhRM8amLECLKxq\nZ/5DT/" +
       "8o9dCJKIosowHTyhgUs0LZsdxSuVCmrO6gm2zLWC8ZFlccQzyOH/qo9vrG3H" +
       "AM9S+jfmrm\nOeZUy1omJ3W+jJIVUlkjDsvoOtGXUcokRM8Th2KDXgBCywTB" +
       "jJZMzF2HsCXCLKMqCAeYaxNHyvQn\ncyipWSbjjqtxy2Ec7cqdxVU86XJqTO" +
       "Yo41M5FC9SYujsPLofRXOou2jgEhDuzflWTEqOkzNiHsh3\nUFDTKWKN+Fu6" +
       "zlFT5+hAcEfD4sMngQC2JiqEl62GqC4TwwQa8FQysFmazHOHmiUg7bZckMLR" +
       "0KZM\ngajHxto5XCKrHO0L0i16S0DVK90itnC0J0gmOcGZDQXOrOW0FuLJfz" +
       "+y+M+b4MRBZ51ohtA/Dpv2\nBzYtkSJxiKkRb+NVN31x9m53JIoQEO8JEHs0" +
       "mSM/vSv3wS8PeDTDHWgWJBZXtY9uHxm9kvlDb0yo\n0WNbjAootFkuT3VRrU" +
       "zVbUD33gZHsZj2F3+19PLdD3yf/DmKemZRXLMMt2LOol5i6lk1TsA4R03i\n" +
       "zS4Ui4zwWdRlyKm4JX+DO4rUIMId3TC2MS/Lcd1GCCWgRaD9HnmfG0TH0Y5s" +
       "GfMCXkuzsxwd4YRx\ntsIcbYVpML+iGZSYfKXk0pUmXV0w3VmLRMCekWBsGQ" +
       "DEE5ahE2dVe+b9V++bPvnVh72TEuhS6oCX\nJP+0xz8N/NOKP4pEJONBAULP" +
       "SRnHwesiOOoPXhn91m/wE+ByMJ3RC0RaFql1iR42fXrL3JFtRt4s\njDAAY1" +
       "Xb/cAHQ99+63svRVGsY/7INSZnLKeCDQEBP2AGlLjgCiDncBC/nWT/9ZH559" +
       "9+7d2bm0jm\n6HAowMI7RYCMBR3vWBrRIQE12T/2rxN/e7T7Mz+Ooi6IOsg7" +
       "HANaIIj3B2W0BcqUn3SELbEc6iuG\nDN/BwUG1FoNFn5TjfjiKHmgQQOh2Bb" +
       "Uh0YnFlOgGPPzAce0O2CBT2tUvxz/1zs/7XpJO8bNff0ua\nzBPuxVKqCZGC" +
       "QwjMv/v44qOXPty4JwbRa9seLDiK2+6aQbU6bLmxPfyEBbpA1l+em9r19Qn2" +
       "E4mC\nXlqpuByvGQTSNTYMq0b0VS7zVaolN8qUBM5KrkFqgyy5agAjzx12pA" +
       "oI7pA80vt2X/zm+HfeEenF\nlv7aK3wlNY3K34McfSIUG/NwcIvYJD6FZ7nQ" +
       "Ij0L2b9EnLpcOhIR/XiDMZKMkXT4KBIeb7oMksk5\nz/jkeP7euTMPj0mvea" +
       "qIDUdFN+ExvjU0XW8Iicu1OHC/ZcvwmxHXWhOd9L5/PPjcm5eSURRdRgnK\n" +
       "ZqiJDeF8dsqLtw4JPcDiwi/u+u7VN/h7EipNgAvFRurh3HQat8TesberqfgP" +
       "n6xEUWIZ7ZKXMjb5\naWy4AkjLcK2yrJrMoRva1tuvSO8+mGoE8EgwuFrEBk" +
       "OrmRNhLKjFuCcQTcLHx6H1q2jqD0RTBCCe\nzE+f+tzq/HQ+n7lzWkwfl9sP" +
       "yH7MO9QYB+HCyRwlbIdWMQdL4kzWOnWO+iSLTLYwu3DKx9muJs68\nSkBGbg" +
       "TJ9dsaGvZCG4aWUhqmQhraYjAd1kni9Q6Q3lNREG9AbCfqJGYPtEElZrCzmD" +
       "nR3clR7BxZ\nl1jdnNs+xRH532Fu84pbV5WSmu+XPcIv9TSrgUvScwWoEjMO" +
       "wZvL2a+OEfnfYTlLSk6PkNMa6ze2\nycprjmUYvqM6SxtSliH/OyztC0paNz" +
       "Vtl/ui9obMkgGnJElYHvTyDGrmmVvV7at4jErI4BpPk6pI\nXxlN1DcizRBT" +
       "pCkHjW5W2smydOOLf09+Bb94r1eADbTn62nTrdz25O/I+B1JrUNR0cste8IA" +
       "uUYj\nOfVK7XpB7MSWyWleVsLN9BDLf3b8k7f0/Qnuzk3KgpSaXCJQ9psFP3" +
       "hlvYCvu144EHBKUJ9Udfjz\nsTL9bVSmDS/ThAr89k1T7fllh9OuqDzO4Tbg" +
       "HIJ2TAHnWKc7W6Jmy5vmmtdQmaNBh5x3odCcsTSX\nLZizEoLhZL/o0AoUx1" +
       "VVvX9j/9N/fP79pd3euXtPnEOhV0brHu+ZI6X22QJ5B7eSIKlfPHrw2fuX\n" +
       "3luLKm2xCHyL6tKg8y14R/XwD9JWAB2FtqCcubCZMw929FekPaSGOobUtBhf" +
       "2981KJ6w3LAIr0QA\nHpHxfD5kkZz4Un2TmXbbRIbByjZ8fbZ9bKA8BFcSMU" +
       "UVJgHSWelrH4O4l8pK1fI2qfo1uIB1yv5v\nXQ9DqyhdK/8TZMTP7LUVvgjv" +
       "Pmzb8JAUSX4LPGRDeMgGFffvtw2l+MY2JY4nONpZsaokiyGRFaxp\ncxMcb+" +
       "1moe0YtEtK20vbpO1lKJ5KhGfgAW8QUQAqWGQ9J143hJ9S+j61TRD+AdyeoC" +
       "88wE+qqkkS\nT1yXY49Au6wUvbxNjv0ZVF9Y1zOmZa5XLJf5lQX7WGiASjOh" +
       "/mgQL8h9oX8Gvf+vtLErZ25+wU69\nIp/Ojf+YEjnUU3QNo7VqbxnHbYcUqd" +
       "Qy4dXwtvx6AbAQfNJBeQq91O3XHtXLkERaqaA29watRK9A\nySaJxI9Xbf+G" +
       "GGgr3ER1aNTRfwHNqpY6UBUAAA==");
}
