package schat.client.gui;

import java.awt.Dimension;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import schat.common.*;
import schat.client.*;

public class ChatToolBar
extends JToolBar
  implements ActionListener
{
    final private static String
      CONNECT_ACTION =
      "CONNECT";
    final private static String
      DISCONNECT_ACTION =
      "DISCONNECT";
    final private static String
      TOOLBAR_CONNECT =
      "Connect";
    final private static String
      TOOLBAR_DISCONNECT =
      "Disconnect";
    final private static String
      TOOLBAR_USER_NAME =
      "Username: ";
    final private static String
      TOOLBAR_SERVER_ADDRESS =
      "SJChat server: ";
    final private static int
      USER_NAME_FIELD_WIDTH =
      10;
    final private static int
      SERVER_ADDRESS_FIELD_WIDTH =
      30;
    private ChatClient
      client;
    private ChatClientGui
      gui;
    private JButton
      connectButton;
    private JButton
      disconnectButton;
    private JTextField
      userNameField;
    private JTextField
      serverAddressField;
    
    public ChatToolBar(ChatClient client,
                       ChatClientGui gui) {
        super();
        this.
          client =
          client;
        this.
          gui =
          gui;
        this.
          setFloatable(
          false);
        this.
          setRollover(
          true);
        this.
          addToolBarItems();
        this.
          enterOfflineMode();
    }
    
    public void
      actionPerformed(
      ActionEvent e) {
        String action =
          e.
            getActionCommand();
        if (action.
              equals(
              CONNECT_ACTION)) {
            String userName =
              userNameField.
                getText().
                trim();
            if (userName.
                  length() <
                  1) {
                gui.
                  showMessageDialog(
                  "Invalid username.",
                  ("Username must contain at least one non-whitespace character," +
                   " not: ") +
                  userName);
                return;
            }
            String[] serverAddress =
              serverAddressField.
                getText().
                trim().
                split(
                ":");
            if (serverAddress.
                  length !=
                  2) {
                gui.
                  showMessageDialog(
                  "Invalid server address.",
                  "Server address should be of the format [host]:[port], not: " +
                  serverAddressField.
                    getText());
                return;
            }
            String host =
              serverAddress[0].
                trim();
            int port;
            try {
                port =
                  Integer.
                    parseInt(
                    serverAddress[1].
                      trim());
            }
            catch (NumberFormatException nfe) {
                gui.
                  showMessageDialog(
                  "Invalid server address.",
                  "Bad port number: " +
                  serverAddress[1]);
                return;
            }
            if (client.
                  connect(
                  host,
                  port,
                  userName)) {
                this.
                  enterOnlineMode();
            }
        } else
                  if (action.
                        equals(
                        DISCONNECT_ACTION)) {
                      client.
                        disconnect();
                  } else {
                      System.
                        err.
                        println(
                        "[ChatToolBar] Shouldn\'t get in here: " +
                        action);
                  }
    }
    
    protected void
      performDisconnect(
      ) {
        this.
          enterOfflineMode();
    }
    
    public void
      enterOnlineMode(
      ) {
        connectButton.
          setEnabled(
          false);
        disconnectButton.
          setEnabled(
          true);
        userNameField.
          setEditable(
          false);
        serverAddressField.
          setEditable(
          false);
    }
    
    public void
      enterOfflineMode(
      ) {
        connectButton.
          setEnabled(
          true);
        disconnectButton.
          setEnabled(
          false);
        userNameField.
          setEditable(
          true);
        serverAddressField.
          setEditable(
          true);
    }
    
    private void
      addToolBarItems(
      ) {
        connectButton =
          ChatToolBar.
            makeButton(
            CONNECT_ACTION,
            TOOLBAR_CONNECT);
        disconnectButton =
          ChatToolBar.
            makeButton(
            DISCONNECT_ACTION,
            TOOLBAR_DISCONNECT);
        Dimension buttonSize =
          new Dimension(
          disconnectButton.
            getPreferredSize());
        connectButton.
          setPreferredSize(
          buttonSize);
        disconnectButton.
          setPreferredSize(
          buttonSize);
        connectButton.
          addActionListener(
          this);
        disconnectButton.
          addActionListener(
          this);
        this.
          add(
          connectButton);
        this.
          add(
          disconnectButton);
        this.
          addSeparator();
        JLabel userNameLabel =
          new JLabel(
          TOOLBAR_USER_NAME);
        userNameField =
          new JTextField(
            ChatClient.
              DEFAULT_USER_NAME,
            USER_NAME_FIELD_WIDTH);
        this.
          add(
          userNameLabel);
        this.
          add(
          userNameField);
        this.
          addSeparator();
        JLabel serverAddressLabel =
          new JLabel(
          TOOLBAR_SERVER_ADDRESS);
        serverAddressField =
          new JTextField(
            ChatClient.
              DEFAULT_SERVER_ADDRESS,
            SERVER_ADDRESS_FIELD_WIDTH);
        serverAddressField.
          setActionCommand(
          CONNECT_ACTION);
        serverAddressField.
          addActionListener(
          this);
        this.
          add(
          serverAddressLabel);
        this.
          add(
          serverAddressField);
    }
    
    private static JButton
      makeButton(
      String actionCommand,
      String text) {
        JButton button =
          new JButton(
          );
        button.
          setActionCommand(
          actionCommand);
        button.
          setText(
          text);
        return button;
    }
    
    final public static String
      jlc$CompilerVersion$jl =
      "2.3.0";
    final public static long
      jlc$SourceLastModified$jl =
      1238676054874L;
    final public static String
      jlc$ClassType$jl =
      ("H4sIAAAAAAAAALVYfWwcRxWfO9vnz8T2xXYsf8V2UrVWmjNBCkpjieRytvEl" +
       "F9v1XVJqHF3Wu3P2\nxnu7y+7s+WKVKCGiCa1AQk0gfIY/LKGiSECrwj+oIN" +
       "FSPgUKUssfpKpIBZWgCP6hESqINx93t7u3\n/ggVlmY9N/Pmvd97896bN3P7" +
       "XVRnWyhmY9tWDf18jFwwsc2+xuJ5LBM7lj4xK1k2VhKaZNsZmMjK\n5/6jnv" +
       "le+6enwig0j6K6EddUyc4sW4aztJxZVu2ihQZNQ7uwpBlEcKzicWTv+6u/un" +
       "qitwa1zqNW\nVU8TiahywtAJLpJ51JLH+UVs2XFFwco8atcxVtLYUiVNXQNC" +
       "QwfBtrqkS8SxsD2HbUMrUMKo7ZjY\nYjJLgynUIhu6TSxHJoZlE9SWOi8VpF" +
       "GHqNpoSrXJWApFcirWFPuT6CIKp1BdTpOWgLArVdJilHEc\nnaTjQN6kAkwr" +
       "J8m4tKR2RdUVgvb4V5Q13ncSCGBpfR6TZaMsqlaXYABFOSRN0pdG08RS9SUg" +
       "rTMc\nkEJQz4ZMgajBlOQVaQlnCer2083yKaBqZGahSwjq9JMxTrBnPb49c+" +
       "3WTKTl38/MvjcIOw6YFSxr\nFH8EFg34Fs3hHLawLmO+8L4Tu5580ukLIwTE" +
       "nT5iThN/6AenU+/8aA+n6Q2gmWG+mJXf/0hf/534\n2401FEaDadgqdQWP5m" +
       "xXZ8XMWNEE7+4qc6STsdLkj+d++uSlb+O/hFFDEkVkQ3PyehI1Yl1JiH49\n" +
       "9FOqjvnoTC5nY5JEtRobihjsN5gjp2qYmqMO+qZEllm/aCKE6qGFoH0H8b8d" +
       "9EPQzsSyRDKGoR2X\nrJh9nqARgm1iL9iWvGDLMLcgayrWycKSoy54aYuU+c" +
       "7VUAj06vPHmAYOOWVoCray8rfu/eKpiZOf\nvcZ3jHqZgEVQH5MR4zJiICPm" +
       "koFCIcZ8F3VIbrC4ZUkXaKAUL9/p//Jr0tfB/GAGW13DTMvQai39\nwqIPb5" +
       "pHEpUoTEJPAifJyh2X3un5yu+efzWMagJzSao8OGlYeUmj7lAKnqgQ558BL9" +
       "rn9+Ug2X97\n5tSLr//y7iMVryZoX1WwVa+kwTLsN75lyFiBZFRh/6V/Tf39" +
       "ubrHXgqjWohAyEFEAs+BgB7wy/AE\nzVgpAVFdalKoOVeleBMBA626FKbfFt" +
       "Zvha1ogAbBhEaF2+2lHzrZTj9R7kOwXR0+HVh6u38l8qE3\nftj8KjNKKRO2" +
       "ulJmGhMeV+0VF8lYGMP43Zuzz9149+onaiCSTZO7BUER01nUVLkIS3Z7Q5Fq" +
       "oFDP\n+usLY22fP2B/n3lBo5rPO0Ra1DCkbknTjFWsZAnLXe2uPMnSExirZR" +
       "HSHGTMrAaMuDnMUAE8OCCR\nxLo7rn9x5Gtv0FRjMnt1UVsxpGH2exdBuz3x" +
       "QWMjwbql+YHA+OE0H3PUIqN7KES/I2UZiMlAzPb9\nNFY6KtaDHLPC7dAykj" +
       "574ty1YWZAjoou2E8/Bzjjg1XDxbKQnWyObu2jm0biJD3tKo6qPvXPyy/8\n" +
       "5kZLGIXnUb1qT6q6pNF9sKd56AXkeR+LtZdPf+P+r8mbzGsqvk6B9RWrU9UZ" +
       "yRWGh18vtEe+eysf\nRvXzqI2d1ZJOzkiaQ31qHk5bOyEGU2iHZ957cvJjYq" +
       "wcy33+OHOJ9UdZJUVCn1LTfoMvsKiND0Nr\nFYHV6gusEHh7fWJmenoikaEj" +
       "R9jKPew7zPezhoBcal+gNC21IBFQImKz6qdIjwa+OhtPZJIz0yWX\na2POQp" +
       "WM8fKAhXAIsflDHnwfhdYm8LUF4GsaT6ZdECcCINL+Ufo5Rj9xgNVeWSSQ0Z" +
       "nExigegxYV\nKKKBVjJ0HTaLjpzcFoTWzMxM6nh8LusCvwmAo9B2CQC7As2g" +
       "2nIFw+PbwhAtYfDacBMYdDc6BIyO\nIBinbWxR1zsySAef2N5ulGCcTk/MZa" +
       "fjpya2QDEFrVOg6AxA0QqHM6SwQcBSwBaHcnZbULpKUADJ\nGQATHx+fm0in" +
       "t8BzCNpugWd3FZ7y0cL8PQml9hK2on/85vp7l68eDtPao65Agx/yiisuph16" +
       "YXj6\n9o3+5utvPcsSPDBuokzlbanSWbZmdjI5kRrPPpEcz0zZ1alv1lLzUE" +
       "EWRIn7hYH1P714b66DVxH8\nHrC3qhR3r+F3AZZbmk16LA5tJoFRv7J/6PbF" +
       "uTcXeY0c9R6jE7qTP3Tr93jkWIscUO/VwG2F7QXU\naNV7cQRat9iL7uq9oB" +
       "0MowO0Y23LkD1eX3Bbk5Lkg7E0QuuF1iOw9FRhMWmnWA2BnbTHQHCEH8nl\n" +
       "o3En8jtfSUivENIbLORT9LMGloPDnZ2xG3PbA61fcOsP5nZJcNsh8s1xhxA4" +
       "xERqZ4V2MWavQlaP\nneBzG4sbplshxA0Ei/uMENemlDNcheuVzVkPCtaDwa" +
       "yvlTRxIFNMg49Nlq/NoEmXR5MM3OPZ9MYS\nH4Y2JCQOBUv8nJAY5akprihw" +
       "3bfLfJ8VzFksDfHCi3EZcdVKoRLAfpYtpFUSwwVausVleg+kdReG\nWoUGYv" +
       "9GV2AWhFc//o+Wp6VXzoaFQIOgRmKYBzRgp5WLsAgTGgFuBzYtwk6xh4BKGV" +
       "STPjry8KPN\nf4brwgY3oXYxOIeJY+mZUpHCrkjSA1+R9vh09eNpL/Q+XrOs" +
       "/izMyiNeUVW9b3gXjXnrqCbLC5QZ\nrddzTdkP7aBwgYM+F2B769pYX0Ud8m" +
       "5uT+DmTtD+1hX5V+EMlNiCWWzR+xbmXr1i8rU6QbUFQ1UY\ni3W3b9HureIG" +
       "IzfLujYjfg1Tha5qkK4BqQ08zLQMAi6Dlc3V2FLH56FwMLl23tpnvUorVKz+" +
       "cdOz\nbzT36EIX/cH2bduAX4JNwfShbUanTy6nDAX/r3Bpli4IuIX/E9yXIe" +
       "VyuLncB8FbOlUuCrwXN8K7\n9sHwvkZ9XlHEu0+S4DwjW0cPBpfeMc5CWxdw" +
       "17fn2ewCK0qGQKjhChmrJBNba/RbqKfz0gp2HXeM\n+or/4uxmWD0MlUSz60" +
       "WMPnV0Vz1n80dXefjOuUd+Yrb/nL3xlB9G61OoIedomvtO6epHTAvnVAa5\n" +
       "nt8wTfbvLniQ/22BFyEM6R841VsEbqQuqnLd4ya6B9dMRkR/vG2WkmSH94jm" +
       "ChbRfwEl05jJBxgA\nAA==");
}
