package schat.client.gui;

import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import schat.common.*;
import schat.client.*;

public class MainPane extends JTabbedPane {
    private ChatClient client;
    private Map tabs;
    
    public MainPane(ChatClient client) {
        super();
        this.client = client;
        this.tabs = new HashMap();
        this.addAnonymousListeners();
    }
    
    public void appendMessage(Integer key, String msg) {
        ChatTab tab = this.getChatTab(key);
        synchronized (tab)  { tab.appendText(msg + "\n"); }
    }
    
    public void addChatTab(Integer key, String title) {
        ChatTab tab = new ChatTab(this, key);
        synchronized (tabs)  {
            if (tabs.containsKey(key)) {
                this.enableChatTab(key);
            } else {
                tabs.put(key, tab);
                this.addTab(title, tab);
            }
        }
    }
    
    private ChatTab getChatTab(Integer key) {
        synchronized (tabs)  { return (ChatTab) tabs.get(key); }
    }
    
    public void enableChatTab(Integer key) {
        synchronized (tabs)  { ((ChatTab) tabs.get(key)).enableInput(); }
    }
    
    public void disableChatTab(Integer key) {
        synchronized (tabs)  { ((ChatTab) tabs.get(key)).disableInput(); }
    }
    
    public void removeChatTab(Integer key) {
        synchronized (tabs)  { this.remove((ChatTab) tabs.remove(key)); }
    }
    
    protected ChatClient getClient() { return client; }
    
    private void addAnonymousListeners() {
        this.
          addFocusListener(
          new FocusListener(
            ) {
              
              public void focusGained(FocusEvent e) {
                  ((ChatTab) MainPane.this.getSelectedComponent()).
                    requestFocusInWindow();
              }
              
              public void focusLost(FocusEvent e) {  }
          });
        this.
          addMouseListener(
          new MouseListener(
            ) {
              
              public void mouseClicked(MouseEvent e) {
                  ((ChatTab) MainPane.this.getSelectedComponent()).
                    requestFocusInWindow();
              }
              
              public void mouseEntered(MouseEvent e) {  }
              
              public void mouseExited(MouseEvent e) {  }
              
              public void mousePressed(MouseEvent e) {  }
              
              public void mouseReleased(MouseEvent e) {  }
          });
    }
    
    final public static String jlc$CompilerVersion$jl = "2.3.0";
    final public static long jlc$SourceLastModified$jl = 1238676054879L;
    final public static String jlc$ClassType$jl =
      ("H4sIAAAAAAAAALVYa2wURRyfu7bX1wG9Up4tFAoIDXA1EAnSRG0aGlqu0hdE" +
       "zpJ2bnfuOmVvd92Z\nux4ECWgCyAeNAXwrMfERDCYKUb8YNRF8GxNMwC/wBa" +
       "MkitEvSgwa/zO7d7e3dxSr0GSmczP/mf/r\n93/cnbyKKpiFwowwRg19PMx3" +
       "mYTJ2YiNE4Wz8GBPH7YYUTs1zNgQHIwoo3/TbW+HHtnkR74oqteN\nDo1iNj" +
       "RmGanE2NAYZRkLLTINbVdCM7jzYtEbG5Zcn/jqYE9jGZoRRTOoPsgxp0qnoX" +
       "OS4VEUTJJk\njFisQ1WJGkUhnRB1kFgUa3Q3EBo6MGY0oWOesggbIMzQ0oKw" +
       "nqVMYkme2c0ICiqGzriVUrhhMY7q\nIuM4jdtSnGptEcp4ewQF4pRoKnsI7U" +
       "X+CKqIazgBhLMjWS3a5IttXWIfyGsoiGnFsUKyV8p3Ul3l\nqNl7I6fx0s1A" +
       "AFcrk4SPGTlW5TqGDVRvi6RhPdE2yC2qJ4C0wkgBF47m3/BRIKoysbITJ8gI" +
       "R3O9\ndH32EVBVS7OIKxzN8pLJl8Bn8z0+c3lrSyD41+G+PxaBx0FmlSiakD" +
       "8AlxZ6Lg2QOLGIrhD74rVU\n+Gj39lSTHyEgnuUhtmk6lr23NXLlw2abprEE" +
       "zRaJxRHl+rqmBec6vq8uE2JUmQajAgoFmkuv9jkn\n7RkT0D0796I4DGcPPx" +
       "r4ZPu+N8hPflTVjQKKoaWSejeqJrra6awrYR2hOrF3t8TjjPBuVK7JrYAh\n" +
       "P4M54lQjwhwVsDYxH5PrjIkQqoThg/Eysv+CYuKothdTvQ/rJMzGObqDE8bZ" +
       "MLOUYaaMYT6saJTo\nfDiRosMuwox4dvqEzwcaNXmjSwMobjI0lVgjyuuXv9" +
       "izcfNjh2xfCXw5AnE0TzII2wzCwCCcZYB8\nPvnyTIFD204dloV3ifjI7D+3" +
       "4NlP8YtgddCe0d1EKuebKBczXFozafrozAdfN6wwYGNEadh3Zf5z\n354460" +
       "dlJVNIJLfZZVhJrAkUZGOm3mHnPQHwLPVCuBTvXw73nj7/5cUVeTBztLQoxo" +
       "pvihhp8Vre\nMhSiQg7KP//0n5t+PVJx9zt+VA6BB6mHg41FHC/08iiIlfZs" +
       "3hG6lEVQbbxI8RoOBppwKSzmoFzP\nAFdUwYAYQvMctNWLSRyGxFRvAwjc1e" +
       "DRQWa1a48G7rzwfu1ZaZRsApzhypSDhNvhFMpDZMgiBPYv\nPtN35NjVgw+W" +
       "QQCbpg0LjgJmKqZRJQNX5hRGoNBAFcj6+VR73eOr2bsSBdU0mUxxHNMIZGys" +
       "acYE\nUUe4TFkhV3qUWQmMFYxBdoNEOaLBQ7Y5TF8aEFwif4TnNhx9qvWFCy" +
       "LDmNJes10AFp9ncjSnIDg6\nYd0plxlJsMwn5tbcZSQvI2nUBSIIGvJmgZyx" +
       "01Yw2Dq4o2f0UIu0jM1OXFhpv9nm3snknvbLbT+8\nuWrSwOoSNSuPO7rn9/" +
       "2nvjkW9CN/FFVS1kV1rAmzsvvtSCqRrT1P7P5g60vXvuaXJAjy0BWCNWWK\n" +
       "08427Iqq9efTocBbx5N+VBlFdbLiYp1vw1pKQCQKNZN1OpsRNK3gvLD+2cm+" +
       "PReaTd6wcbH1Bk0+\n3cFaUIt1lSdOqmE0wpjuxMl0T5z4fKZYrJcXmuXckk" +
       "N1pWnRNBZNCtQMiQ/pPhlbCEnitQWcRBTW\nOZzqSnO6R0wbOEiMYywLx2l5" +
       "PPVi03lfarLYBiTKA7LNASNAZsGNirhsQA4+8FvwAD6zwy619YVh\nuVFPJe" +
       "86/h1pvS+olCge1dwwV2skTbQcUqsk3ypgu3pSpPbKniePlbLBe1uXr6r9EV" +
       "LkDbJ/yNkc\nINDg6UNZT8qygKdcFpo9RvHKE0o39peN0c/8EkM27IpaucJL" +
       "7YVgq7EKBZWOaixIzWthtDhAaCmV\nml2+9SSbbEZwoGFnYBEs4W5oRBPQJz" +
       "sndfkTu428efLqBahh04QGpxfsB+0iK04TfRZNQs+Udpq6\nJxe+8sPpywMN" +
       "NkjszndJUfPpvmN3v5JfrSlgungyDpL6zMrFJ/cOXIr5HTm7ID7SBlWlKlEX" +
       "7P1i\n2S+mwcxk2z0F3lgDY53jjXX/yRve9ycz8ihHNVhVRVEZwjGpAfr/eo" +
       "j0sgJGj6NHz4302FBSQF++\n/PTfXAUKKiQId1TIAm5uUUfpEMiXTG+Z6y8q" +
       "fP2lnLMcxoCj1MDUnDM1pTiAn+ii53C7pghiU5Fd\nOCTqyB69nbI/zNF0lb" +
       "JbKrwwfMwRPnY7hT8AhrdI0kjfGtlrYSyGMe7IPl5K9pL1vNq0DA5liqiT\n" +
       "C31TjZ6At0R85HsCu7ErLNBFHwqjeRkMw9HBmFo0/2tBn+doFuSiDt3QdyWN" +
       "FBN9KtGdGlLsAo/Q\n0PdUZb80im8Dc4t+6LF/jlBazo2u+NgMfS6/BuV+Mq" +
       "iE7+3xlKa5+zTXOmBaJE6lmJV212bKf69B\nafNmGo7KYJbCvWpTneDQ5bmo" +
       "ci2am+gkRxWSSHx408x1/6JyZsJsAopmuAcAGSOq0DGD/gGhJeV4\nJBMAAA" + "==");
}
