package schat.common.events;

import java.util.*;

public class PrivateConversationEvent extends ChatEvent {
    private Integer targetId;
    private Integer lock = new Integer(0);
    private Boolean[] accepted = new Boolean[] { null };
    private String hostAddress = null;
    private Integer port = null;
    
    public PrivateConversationEvent(Integer userId, Integer targetId) {
        super(userId);
        this.targetId = targetId;
    }
    
    public Integer getRequestorId() { return this.getUserId(); }
    
    public Integer getTargetId() { return targetId; }
    
    public void accept() {
        synchronized (lock)  {
            accepted[0] = new Boolean(true);
            lock.notify();
        }
    }
    
    public void reject() {
        synchronized (lock)  {
            accepted[0] = new Boolean(false);
            lock.notify();
        }
    }
    
    public boolean accepted() {
        synchronized (lock)  {
            while (accepted[0] == null) {
                try {
                    lock.wait();
                }
                catch (InterruptedException ie) {  }
            }
            return accepted[0].booleanValue();
        }
    }
    
    public String getHostAddress() {
        synchronized (lock)  {
            while (hostAddress == null) {
                try {
                    lock.wait();
                }
                catch (InterruptedException ie) {  }
            }
            return hostAddress;
        }
    }
    
    public void setHostAddress(String hostAddress) {
        synchronized (lock)  {
            this.hostAddress = hostAddress;
            lock.notify();
        }
    }
    
    public int getPort() {
        synchronized (lock)  {
            while (port == null) {
                try {
                    lock.wait();
                }
                catch (InterruptedException ie) {  }
            }
            return port.intValue();
        }
    }
    
    public void setPort(int port) {
        synchronized (lock)  {
            this.port = new Integer(port);
            lock.notify();
        }
    }
    
    public String toString() {
        return "<Private conversation event: " + this.getUserId() + ", " +
        this.getTargetId() + ">";
    }
    
    final public static String jlc$CompilerVersion$jl = "2.3.0";
    final public static long jlc$SourceLastModified$jl = 1238676054826L;
    final public static String jlc$ClassType$jl =
      ("H4sIAAAAAAAAALVYe2wbRRof24mdh9vEadorTdI0bXhEbZ3qeAgaCS4XNWqK" +
       "afOiglwgHa/H9jbj\n3WVn7GwrVJU76crxBxUqcAfi8QdIPMQfUARIcLo7ic" +
       "e9QEhFahFS+w+IQ7rjBP9w1Yk78c3M2l7v\n2olSEUsznp35vvl+88332n3l" +
       "a9TMbJRkhDHdNI4m+TGLMNmb6aNE4yw5c2AS24xkxihmbBYWFrQj\n/9cPv5" +
       "b45f4wCs2hLsMcpTpms3nbLObys3mdOTbaZpn0WI6a3N0xsMfeHd8vfXjqQE" +
       "8EdcyhDt2Y\n4Zjr2phpcOLwORQvkEKa2Gw0kyGZOZQwCMnMEFvHVD8OhKYB" +
       "gpmeMzAv2oRNE2bSkiDsYkWL2FJm\neTKF4pppMG4XNW7ajKPO1FFcwsNFrt" +
       "PhlM74SApFszqhGXYfOoHCKdScpTgHhJtS5VMMyx2Hx8U8\nkLfpANPOYo2U" +
       "WZoWdSPDUb+fo3LiwduBAFhjBcLzZkVUk4FhAnUpSBQbueEZbutGDkibzSJI" +
       "4WhL\nw02BqMXC2iLOkQWONvvpJtUSULVKtQgWjjb6yeROcGdbfHfmua1D0f" +
       "j/Hpr8zza4ccCcIRoV+KPA\ntNXHNE2yxCaGRhTj5WLy0Ym7i71hhIB4o49Y" +
       "0Yxe/dadqa/+2K9oeurQHJK2uKB9f1Nv37nRL1oj\nAkaLZTJdmELNyeWtTr" +
       "orI44F1r2psqNYTJYX/zT9wd0nXyb/DKOWCRTVTFosGBOolRiZMXccg3FK\n" +
       "N4iaPZTNMsInUBOVU1FTPoM6sjolQh3NMLYwz8uxYyGEYtBC0M4g9RPPiKOe" +
       "SVsvYU7A2ktg49Ke\n95WIwZPsKEe3cMI4m2e2Ns+0PObzmlkomMY8ERRsfh" +
       "leR0hevxQKwaF7/Q5IwVr3mzRD7AXthc//\ndv++23/zoLpOYYIuZo52SZlJ" +
       "JTOpZCYbyUShkBS2QViv0u6obeNjwqucB871PfFn/DTcFeiM6ceJ\nVEloqU" +
       "n0wPTTZYPOWNVlJ2CEwaIWtO6TX2158pOX3g+jSN3Ak6pMjpt2AVNhO2VP63" +
       "LF+VfA5Ab9\nhl9P9r8fuuP183+/eF3VBTgaDHhmkFN41nb/ZdimRjIQuarb" +
       "//a/+78503zLG2HUBO4KAYtjMDPw\n/q1+GTUeNlKOVuIskRRqzwYO3sZBQU" +
       "ueA4s+LscdcBUt0MDz0BbXRrtFJxYToutSNgXX1e07g4yF\nl38V3XPh9+3v" +
       "S6WUw2aHJ77OEK6cMFE1kVmbEJi/+LvJM499feoXEXB7y1JmwVHUKqaprjnA" +
       "8pNa\nvxUnyAjL+tfZkc6Hd7M3pRW06oVCkeM0JRDnMaXmEskscBnoEp6gKm" +
       "MZKCuehpgI4XWBwkZKHVao\nBBZcJ+okN3c/+vjQUxdEXLKkvjYJXUmkYfm8" +
       "gbvnEjKSE5AUcsQWCzsduX51SPRDFV4keZHUaZ/w\nge6qViDQLKrzxYdm7j" +
       "lw5MHtUjFKmtzTs/FwYNqpCGmWayKt71rWw8ZFyqsaoH7/dw+c/fixeBiF\n" +
       "51BMZ+O6ganQLzuoXKpOsPdtcfwPdz5z+SN+SVpD1YYFsF4nGJIOY4973Xy+" +
       "lIi++mwhjGJzqFMm\nbGzww5gWha3MQcplY+5kCq2rWa9NnypXjFR8tNfvPx" +
       "6xfu+phkIYC2oxbvE5TKvrLK2uw7T6HCYU\nssTgZsnQL/vtFfOOWSqUOhy1" +
       "cGznIKFkKre4HiFJfn2NrGuhrXNlrasv61bR7eWQmkxtcYXtboTW\n4W7XUX" +
       "+7UXe7FqxpxOIkw4LFgYzzKn+fjQx+G35n06AMXU1pzJTa/FVVsGiqqYWklt" +
       "tqoHYuA7WO\n//3cNCnBRsURyqlGPN8gunEnhOQBDwTvJszBFITJi7lBuJ4o" +
       "JUaO51nQ6iEbFqCCKLklziNbn//y\n9c+nu1ViUHXgjkAp5uVRtaA8cLslIt" +
       "3AchIk9Xs7B145MX0prWqkrtrIuM8oFm589lMy9LO4Viel\nR0Dx4mGf0yiI" +
       "bagqcYyaBhHhtLymQpRuJivlNyw6ASXbqN93iDvk5VY9PFHqmYrk9b+Epacp" +
       "5wzU\ny7VMI7Uu2WYTKPeN2RrH7FE3Kw8ouoFlI++KYflesANN6KB8/s6qbl" +
       "RocRr71g5oCddgE/V9K+v6\nVnveZBzebuDthdURpV4CGkvqg9blSuqqL2mx" +
       "HBQs0+beoOB41ISqmhguq8hGfY3eBKQtnrrr2/iv\n8Xv3hN2tZjhq5aa1m0" +
       "K1SCtKbZPbtcFuu5dNQ34ziczcNnTtrvZ/QDRpUOMl3MnpWmuQxR9eZfEn\n" +
       "+vmaWmgbtF5Xs731aqEfwciKHK2H0D9N7itCrQ+1osoBkmNn7WUEHngNWmEH" +
       "Ay7agTVCewKsFdDO\netPVFUDdCO0aF+o1awT1FMRtlbTk8iFLsUyDF5RMPS" +
       "M5H1416j0u6j1rhPoMoLaJcAiJ7wpQinpk\nr4ty7xqhfMpbEPi0G0ur3CvZ" +
       "n1sV9H5oB13oB9cI+ovK3/ZXQ66Y1SWHviq0w9DSLtr06tC6mVLK\nXBnyaw" +
       "CZBSAHbcO7X3AmaMvUBU/XSNVvgzGAqifdnDMlSadWpeOt0E66ME9esY6nVs" +
       "b6LmBlVawN\nlDsVUO5UPdRXQTvtoj69Rsr9ULw1mNXaYEULhjp2c6OPN+Kt" +
       "fHPgM636mKhtP3fkunetxF9VTV/+\n4BdLoZZskVLva5JnHLVsktUl0ph6ab" +
       "Lk3wWoMOt8VoKopwYS8nlF+xmHVy0PLRef5sTAS3QRijRJ\nJB4uWeX6qa/e" +
       "t6sxmJLnddAPDLh/oO4WAAA=");
}
