package schat.server;

import java.util.*;
import sessionj.runtime.*;
import sessionj.runtime.net.*;
import util.*;
import schat.common.*;
import schat.common.events.*;

public class ChatServer {
    private ChatProtocols dummy;
    final public static Integer CHAT_SERVER_ID = new Integer(10);
    final protected static int USER_ID_START_VALUE = 101;
    protected static boolean debug;
    private SJSessionParameters params;
    private int id = USER_ID_START_VALUE;
    private Map users = new HashMap();
    
    public ChatServer(String setups, String transports, int port_s)
          throws Exception {
        super();
        this.params = TransportUtils.createSJSessionParameters(setups,
                                                               transports);
        SJServerSocket ss_s = null;
        try {
            ChatServer.debugPrintln("[ChatServer] Server starting...");
            ss_s = SJServerSocketImpl.create(ChatProtocols.p_sc_chat, port_s,
                                             params);
            ChatServer.debugPrintln("[ChatServer] Server listening on port: " +
                                    port_s);
            while (true) {
                SJSocket s_sc_main = null;
                try {
                    s_sc_main = ss_s.accept();
                    ChatServer.debugPrintln(
                      "[ChatServer] Connection accepted from " +
                      s_sc_main.getHostName() + ":" + s_sc_main.getPort());
                    {
                        SJSocket _sjtmp_s_sc_main = null;
                        _sjtmp_s_sc_main = s_sc_main;
                        s_sc_main = null;
                        new ServerInThread(this).spawn(_sjtmp_s_sc_main);
                    }
                }
                catch (SJIncompatibleSessionException ise) {
                    ChatServer.debugPrintln(
                      "[ChatServer] Incompatible client: " + ise);
                }
                catch (SJIOException ioe) {
                    ChatServer.debugPrintln("[ChatServer] I/O error: " + ioe);
                }
                finally {
                    SJRuntime.close(s_sc_main);
                }
            }
        }
        finally {
            if (ss_s != null) ss_s.close();
        }
    }
    
    protected void propagateEvent(ChatEvent e) {
        synchronized (users)  {
            for (Iterator i = users.keySet().iterator(); i.hasNext(); ) {
                ((ServerOutThread) users.get((User) i.next())).queueEvent(e);
            }
        }
    }
    
    protected void sendEvent(User user, ChatEvent e) {
        synchronized (users)  {
            ((ServerOutThread) users.get(user)).queueEvent(e);
        }
    }
    
    public static void main(String[] args) throws Exception {
        boolean debug = Boolean.parseBoolean(args[0]);
        String setups = args[1];
        String transports = args[2];
        int port_s = Integer.parseInt(args[3]);
        ChatServer.debug = debug;
        new ChatServer(setups, transports, port_s);
    }
    
    protected void addUser(User user, ServerOutThread sot) {
        synchronized (users)  { users.put(user, sot); }
    }
    
    protected User findUser(Integer key) {
        synchronized (users)  {
            for (Iterator i = this.getUsers().iterator(); i.hasNext(); ) {
                User user = (User) i.next();
                if (user.getId().equals(key)) { return user; }
            }
        }
        return null;
    }
    
    protected Set getUsers() { return users.keySet(); }
    
    protected void removeUser(User user) {
        synchronized (users)  { users.remove(user); }
    }
    
    protected Integer getFreshId() { return new Integer(id++); }
    
    public SJSessionParameters getSessionParameters() { return params; }
    
    public static void debugPrintln(String m) {
        if (ChatServer.getDebug()) { System.out.println(m); }
    }
    
    public static boolean getDebug() { return debug; }
    
    final public static String jlc$CompilerVersion$jl = "2.3.0";
    final public static long jlc$SourceLastModified$jl = 1238676054770L;
    final public static String jlc$ClassType$jl =
      ("H4sIAAAAAAAAALVZe2wcRxmfu7Pv/Mrj7Lxtx3k4NMbNua1ECJiX5drKJefk" +
       "uLMt4qY9r3fnzpvs\ni93Z8yVUbQOoaYoSCTUJj5Q2f1QioAhBqoIqEFSiBc" +
       "pDoIBa/kmFKCqVoED/oVFVEN/M7N7dPu7s\nBLA0e7Mz33zzPX/zzfrqW6jV" +
       "MlHKwpYl69qxFDlhYIs99YVjWCRWKn8gK5gWlsYVwbKmYaIgzv9b\nnv1O8r" +
       "P7oygyh7o1fUyRBWt60dTt0uL0omxVTLTN0JUTJUUnDscAjw/vfG/pl6cP9M" +
       "bQmjm0Rtby\nRCCyOK5rBFfIHOpSsbqATWtMkrA0h5IaxlIem7KgyCeBUNdg" +
       "Y0suaQKxTWzlsKUrZUrYbdkGNtme\n7mAGdYm6ZhHTFoluWgStzRwTysKITW" +
       "RlJCNbZDSD4kUZK5L1afQQimZQa1ERSkC4IeNqMcI4jkzS\ncSDvkEFMsyiI" +
       "2F3SclzWJIIG/CuqGg8eBAJYmlAxWdSrW7VoAgygbi6SImilkTwxZa0EpK26" +
       "DbsQ\ntKUhUyBqMwTxuFDCBYI2+emyfAqo2plZ6BKC1vvJGCfw2Rafz+q8dT" +
       "je9a/Hs+9sA4+DzBIWFSp/\nHBZt9S3K4SI2sSZivvCmnTqfPmL3RREC4vU+" +
       "Yk4ztut7M5k3fzTAaXpDaA6zWCyI7+3t678+9qf2\nGBWjzdAtmYaCR3Pm1a" +
       "wzM1oxILo3VDnSyZQ7+ULuJ0ce+Sb+SxS1pVFc1BVb1dKoHWvSuNNPQD8j\n" +
       "a5iPHi4WLUzSqEVhQ3GdvYM5irKCqTlaoW8IZJH1KwZCKAEtAu0q4n9J+iBo" +
       "1fiiQCCay9hMWccI\nGiTYItZRyxSPWiJMHbXY3FEPWYWyXb0UiYBGff7sUi" +
       "AU9+uKhM2C+PXXf/7gxMEzj3Ff0fhyBCJo\nI2Of4uxTNfYoEmF8e2gUciuN" +
       "maZwgmZH5dT1/q/8VPga2Bx0t+STmKkWWWqhT1h0T1PwGK+lXhp6\nAkRGQV" +
       "z3yJtbvvrbb7wURbFQAMlUByd1UxUUGgNuxnQ72/lnIHQG/QEctvffHp969p" +
       "Vf3NhdC2Vw\nQCDDgitphuzw293URSwBAtXYf+nd/f94ovVDz0VRC6QdAA8R" +
       "IFwgi7f69/BkyqiLOlSXWAZ1FgOK\ndxAw0FKdwvTZxfprwBVt0CCDUNqJte" +
       "30QSeT9NHNwwfctc6nA8O0m5+L3/XqDzpfYkZx4W9NHU7m\nMeHJlKyFyLSJ" +
       "MYzf+HL2iQtvnb4vBulrGDwsCIob9oIiixVYstGbf1QDiUbWX6+Nrj23x/ou" +
       "i4J2\nWVVtIiwoGPBaUBR9CUsFwgArWQeODJPAWF0LgG0AkwUFGHFzGJEyRH" +
       "AIeqQ2rTt/cejJVym+GMxe\nG0DGGJM0xt574FBgetE9UhyA6fiwFQS4rCmr" +
       "gB9lB+C+uPWZN559PbeOhxM/BXYGgLh+DT8JmD86\nDWqf7c12YNQvDm+/+l" +
       "DutQWOkN1ee05otvqBp3+Phz7RJYbkfAzOqgrTcleEPoeqFnBT2LFAT80C\n" +
       "ExURGxQjmZT9iIZNze8Aice5B7uG8vcfmH9sB3M9tyczW/UxwnfeFz5XqYqS" +
       "YAQJ2OfOpmgySY/p\nWrLJD/7z1LVfX+iKougcSsjWpKwJCo0l6xCHj5ADys" +
       "fi5A9nnrr5K/Ias1wtX6lgfZUg0s4KdVCy\n75VyMv7tp9UoSsyhtazIEDQy" +
       "Kyg2zYs5KBOscWcwg1Z55r1HPj/fRqt41OcPoLpt/UhR8zb0KTXt\nt/nAoR" +
       "1aP7QeBxx6fOAQiRi0M8YWDLDnjmoqJwxTLgu0LkOtkq2qJ9yI2cKPE1FXVV" +
       "1jxwkAItHh\nKGUUqxFivx+pirEe2secX+T+BsVIB8WIwd5F6l06NgjwYrF6" +
       "EURaPb5/bLqQn8jNTuQK6Xtd2ZK1\naE5DsVbiLl0d8Yu0CdpeaBsdkTYGRK" +
       "qCXj2z7j9efuadU6f3Remp2FqmLoVoqQORQzatXx+9eqG/\n8/wfvsCgBxgz" +
       "5+RC1KP9KYLaDTAghAKW6MAhUK9nJs/0KuSnx3LThdmxzMwEyx5u4YhPnW5o" +
       "26Bt\ndtTZHG7h+4IiRGn/iLttq4QXbB5jewyep3dBJCzouoIFLXxvGmSD0H" +
       "qdvXvD9xbpY5yeEIIpqJbr\nsN3VxDdtjcgqTmmYQMrn+XCWEmOohxuEFt38" +
       "fU6UI/c3uPmis3lUlppYkTLbCW3AYTYQzuy4w6zV\nhoqqqsiqGlJOCYYjLc" +
       "vH7RyGUQ2GOTAiegr0N6q+2Qlw+lNvdz0qvHh/1GF1N8QK0Y09Ci5jpQqj\n" +
       "nYxdJ3Db0xRGp9gdpAZksfzHh+64s/PPULQ0qMeSzmAOw4VLm3ZhhhVqwi0X" +
       "agM+Xf3yJMu9n4wt\nyj+LMoDjmBi4WnkXjXqRsMP0CsqM1lt1cCe0YWgHHQ" +
       "cfDC2WgmDIcmRFp2m/BxvBSxpYnkLkBO2G\ns0BONND3E4BsAAaGUALkZWv8" +
       "ydhS1mWJcfh8XSyxgYcrDUbKHgvcDW3WscBsmAXo4zOhokY5ZlTR\n1qPtjM" +
       "XR9uHl1TwDcWzB5YppyHQJaMSA6WxArcBwTTfa7oD2gKPbAyvzLuM4yBGwqY" +
       "uDV2V2W+IF\n4bXY4NvR728YZBeAlgXBcjPF+40h+AnB82WAydhR1WizA0LN" +
       "NBoODcWP0seTlQhiSPRUA62nmOqA\n+3EFayV+hWXYeKnSyOV15eK4omuYlu" +
       "7uHC8WZT1V/WQDk5VQ+Za4fGyvOoxsEDLLxtMVQGORihNS\n1/Mqq9LUt/R1" +
       "L+N0HjJMhVowPCgjdcYJjnjT7IPQVMdx6m2lGQt1V6M+zyWeX+AP2wTgGgvS" +
       "8hZ6\nDo5xQZLcHG2Wb8+H59vzYUruors4Si7dmpJ1ps8uL/8LBLVBJVhV4G" +
       "ykZp19AU7BEa/YNLHOOGKf\nuTWxVxyTL4PEJUxmGhQJcFVgi3/jrQgCL0F7" +
       "n3MEP3fb9j67vPS/I/QwVfUybhwy9byCI17BKYxd\ndAS/+H+y+A2QGSw+aW" +
       "JrMc0KvSwPgRWbmH5NuQfaJUfSS40k/S/x6g0AS5A0tMgtsnXFFctM2/uh\n" +
       "XXZkvhwm8+0fe/R1eHmN/k5QF7s8ZE044pRm+DkciJjhMJ3ol6wrjk5X/kc6" +
       "rdhB7/LkvZdqRN/n\nGe18Y6fAIdpR+7JKP5ltCvwvhH+xF3dcn9/9YyP5Mi" +
       "8V3K/qiQwAnK0o9ff6un7cMHFRZrIl+C3f\noD+RBNi9/mSgd2TWodJF4pyo" +
       "A45HRkRfOllJ8K0K+g8IUd0+BhoAAA==");
}
