package benchmarks;

import java.io.*;

public class BinaryTree implements Serializable {
    private int value = 0;
    private BinaryTree left = null;
    private BinaryTree right = null;
    
    public BinaryTree(BinaryTree left, BinaryTree right, int value) {
        super();
        this.left = left;
        this.right = right;
        this.value = value;
    }
    
    public BinaryTree(int value) {
        super();
        this.value = value;
    }
    
    public boolean isLeaf() { return this.left == null && this.right == null; }
    
    public void inc() {
        this.value++;
        if (this.left != null) { this.left.inc(); }
        if (this.right != null) { this.right.inc(); }
    }
    
    public void printNode() {
        if (this.isLeaf()) {
            System.out.print("leaf=" + this.value);
        } else {
            System.out.print("node=<" + this.value + ">");
        }
    }
    
    public void print() {
        if (this.isLeaf()) {
            System.out.print("leaf=" + this.value);
        } else {
            System.out.print("node=<" + this.value + ", left:(");
            this.left.print();
            System.out.print("), right:(");
            this.right.print();
            System.out.print(")>");
        }
    }
    
    public void println() {
        print();
        System.out.println();
    }
    
    public static BinaryTree createDepth(int depth) {
        BinaryTree left = null;
        BinaryTree right = null;
        BinaryTree res = null;
        if (depth == 0) {
            res = new BinaryTree(0);
        } else {
            left = createDepth(depth - 1);
            right = createDepth(depth - 1);
            res = new BinaryTree(left, right, 0);
        }
        return res;
    }
    
    final public static String jlc$CompilerVersion$jl = "2.2.3";
    final public static long jlc$SourceLastModified$jl = 1238690426865L;
    final public static String jlc$ClassType$jl =
      ("H4sIAAAAAAAAALVXX2wURRifu+tde9er/UOBAqUtpUSQ9IoPErEv1ELDwWHP" +
       "FlAOyDG3O3ddure7\nzM5dr6ioMQHkwUSL/xKFFxMSw4ORqC9GTcD/mpg+gC" +
       "+QGIwxUYg+GInB6Deze7e3eweEhzaZ6dzM\nN9/3zff9ft/MnruOgiZFfYau" +
       "zuZUncXYrEHMWBJTk8ijKjbNXTCRlh5ZfWvmu+PbVwRQawq1Ktok\nw0yRRn" +
       "WNkRJLoWie5DOEmiOyTOQUatcIkScJVbCqHAFBXUuhDlPJaZgVKDEniKmrRS" +
       "7YYRYMQoXN\n8mQCRSVdMxktSEynJkNtiUO4iIcKTFGHEorJhhMolFWIKpuH" +
       "0VHkS6BgVsU5EFySKJ9iSGgcGuPz\nIB5RwE2axRIpb2mYVjSZoV7vjsqJB3" +
       "aAAGxtzBM2pVdMNWgYJlCH5ZKKtdzQJKOKlgPRoF4AKwwt\nv61SEGoysDSN" +
       "cyTNUJdXLmktgVRYhIVvYWixV0xoKlG03JOzqmyNh6L/nkz+3ecXPstEUrn/" +
       "QdjU\n49k0QbKEEk0i1sabhdip+N5Ctx8hEF7sEbZkRtZ8tDvx66e9lsyKOj" +
       "LjmUNEYmnp1sbulfMjP4cD\n3I0mQzcVDgXXyUVWk/bKcMkALC6paOSLsfLi" +
       "ZxNf7H3uXfKbHzXFUUjS1UJei6Mw0eRRe9wI44Si\nEWt2PJs1CYujBlVMhX" +
       "TxG8KRVVTCw9EAYwOzKTEuGQihRmg+aDKy/vhvxFDLo4qG6ewuSkjMPMRQ\n" +
       "PyMmM/ebVNqfgdBN5TGdNve7hEpcacuMzwfn6fZySwUgbtNVmdC0dPbaN09v" +
       "3fHiCX8FXbY7kHhH\necxRjnw+oXWpO0o87DJnx+/vD7e9NGh+6EeBFAor+X" +
       "yB4YxKgFVYVfUZIqeZgFV7FYQFcgB20Qwg\nEMCcVkGRQDyEpUhRvxdpDj/j" +
       "MMIAn/mj//1wIz1znoOCJ7GTa7dcg5RMW75F100e2H7wRH+AC800\nQHD9ID" +
       "pwd+1p6cbJnecvfXtlrYNohgZqiFa7kxPF636S6hKRoRA56l//Z9sfc8FNH/" +
       "g5KsJQfxgG\n1ACZe7w2XIQZLhcfHix/AjVndZrHKl8qV4wIm6L6jDMjkNEs" +
       "xvdBAJp4EKB12JBr4R1fbONdu4Uj\nHk/PGURpu/lCaMPlj5s/91dXwdaqcj" +
       "lJmMWpdicdHEQwf+WN5Nyr14/vE7mwkuFjKGQUMqoilYR/\ni32Q+0V1+B3r" +
       "6jz12rq3LpeTvcjRPkIpnuW5Lj0/v/LNL/HbwH3goKkcIYJiAWEpII5VPpu3" +
       "JiWp\nkgfKF+2a9HLPO7+cvzbR6a8q3Ktramf1Hqt4i9hFjBJYWHUnC0L64v" +
       "pV545OXM1YRa3DTa6tWiH/\n0JkfybrNUakOUQNwvZQjxvsHxHg9z6Q4LRKe" +
       "9InZ7prML7Uz31kv87wbqKvaZ6WM/xy6u/GNtghn\ns7cejfG7tEyFfOapvy" +
       "6cjvRZqeV7llsZq5Mn18a0dOST3advfs+uigg5HOI6eku1ZvfgKno/fKnY\n" +
       "HnrvTN6PGlOoTdz/WGN7sFrgWE3BDW6O2pMJ1OJad9/G1tUzXKkR3V6gVJn1" +
       "stfJKoy5NB83eggb\ntlMWttMW9qTNhww+2CI29It+TYVejQZVipg/mVCwyJ" +
       "0X6bN4wPvNLjN90CK2mUh9M9t5N8bgkiNZ\nVkWqWmWroDXbyprrK9tpKwtS" +
       "JTdVpa1UhULkYMwChk+MO5hd8RU9Vnn3wbXDybfydu8UQbzjT/4Z\nPYYvHv" +
       "DbSH2QoTDTjUGVFInqgNarZKd4lpUz2fpE709jG88+40VtCMz33nFnWmovrn" +
       "g8MKV8BRdm\nBTU170L3pmE3ViKUwLNW2+VCTI+L6F3QltnRX3ZvRC/T+K4c" +
       "PwDlWzETBGfF8qBhbdkAsMvoukqw\nJn7vcznG750e27GeBXIsJ0qk5PWqoa" +
       "grch2XeKwGbZcGF8ilw4Ay4KLGHtNlkbR6seEVeZPtyKYF\ncoRXAuHI7ZxY" +
       "Ai1uOxFfICeetSqTxlStnhut0DZAG7fdGK/nRm298/Px/QBKU3wr1nfj3q6x" +
       "Yww1\nS5RABd1CDOvl3gIRjDjPY/4c6ar5mLU+uaT++YNrLxjtX4tXXuWzKA" +
       "TfJtmCqlZX/6pxyKAkqwjz\nIesuMMS/V8Cu80LnM3NGuRa2iVrIL6SYdSGV" +
       "/gdPAbCeaA8AAA==");
}
